// genericInstaller -- update windows and office
//
// Copyright (C) 2008
// Wolfgang Formann
//
// this file is part of the programm genericInstaller.  This program is
// free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your
// option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef NDEBUG
#include <iostream>
#endif

#include <Qt/qlayout.h>
#include <Qt/qpushbutton.h>
#include <Qt/qevent.h>
#include <Qt/qapplication.h>
#include <Qt/qtabwidget.h>
#include <Qt/qtextedit.h>
#include <Qt/qtextbrowser.h>
#include <Qt/qlabel.h>
#include <Qt/qfile.h>

#include "AboutProgramm.h"
#include "MyWindow.h"

AboutProgramm::AboutProgramm(MyWindow const *parent)
    : QDialog(),
      _parent(parent)
{
    setWindowFlags(Qt::Window);

    setWindowTitle("ber dieses Programm");
   	setModal(true);

	_ok = new QPushButton("&OK");
	_ok->setDefault(true);
    connect(_ok, SIGNAL(clicked()), this, SLOT(closeForm()));

	_layoutTop = new QHBoxLayout();
	_topGraphic = new QLabel();
	_topGraphic->setPixmap(QPixmap(":/images/small-logo"));
    _topLabel = new QLabel("Windows-Aktualisierer");
    _topLabel->setWordWrap(true);
   	QFont f(_topLabel->font());
   	f.setBold(true);
   	_topLabel->setFont(f);
   	_layoutTop->addWidget(_topGraphic, 0);
   	_layoutTop->addWidget(_topLabel, 1);

	_tab = new QTabWidget();
	_aboutWidget = new QWidget();
	_aboutLayout = new QVBoxLayout();
    _label1 = new QLabel("Windows aktualisieren");
	_label2 = new QLabel(" 2008 Wolfgang Formann");
	_label3 = new QLabel("Logo:  Axel Springer AG. Mit freundlicher Genehmigung.");
	_aboutLayout->addStretch(1);
   	_aboutLayout->addWidget(_label1, 0);
   	_aboutLayout->addWidget(_label2, 0);
   	_aboutLayout->addWidget(_label3, 0);
	_aboutLayout->addStretch(3);

	QHBoxLayout *_qtLayout = new QHBoxLayout();
    QPushButton *_aboutQt = new QPushButton("ber Qt", this);
    connect(_aboutQt, SIGNAL(clicked()), qApp, SLOT(aboutQt()));
	_qtLayout->addStretch(3);
   	_qtLayout->addWidget(_aboutQt, 0);
   	_aboutLayout->addLayout(_qtLayout);

    _aboutWidget->setLayout(_aboutLayout);
	_tab->addTab(_aboutWidget, "ber");

	_authorText = new QTextBrowser();
	_authorText->setReadOnly(true);
	QFile author(":/text/authors");
	author.open(QIODevice::ReadOnly);
	_authorText->setHtml(author.readAll().data());
	_tab->addTab(_authorText, "Autoren");

	_licenseText = new QTextEdit();
	_licenseText->setReadOnly(true);
	_licenseText->setLineWrapMode(QTextEdit::FixedColumnWidth);
	_licenseText->setLineWrapColumnOrWidth(80);
	QFile license(":/text/gpl");
	license.open(QIODevice::ReadOnly);
	_licenseText->setPlainText(license.readAll().data());
	_tab->addTab(_licenseText, "Lizenz");

	_layoutButton = new QHBoxLayout();
	_layoutButton->addStretch(1);
	_layoutButton->addWidget(_ok, 0);
	_layoutButton->addStretch(1);

   	_layoutAll = new QVBoxLayout(this);
   	_layoutAll->addLayout(_layoutTop);
   	_layoutAll->addWidget(_tab);
   	_layoutAll->addLayout(_layoutButton);
}

AboutProgramm::~AboutProgramm()
{
}

void AboutProgramm::closeForm()
{
	QCloseEvent event;
	QCoreApplication::sendEvent(this, &event);
}

#include "AboutProgramm.moc"
