// genericInstaller -- update windows and office
//
// Copyright (C) 2008
// Wolfgang Formann
//
// this file is part of the programm genericInstaller.  This program is
// free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your
// option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef NDEBUG
#include <iostream>
#endif
#include <unistd.h>
#include <errno.h>

#include "DataReader.h"
#include "InstallData.h"
#include "MyWindow.h"

#include <Qt/qdir.h>

#ifdef Q_OS_WIN
#	include <malloc.h>
#	include <windows.h>
#	include <io.h>
#	include <tchar.h>
#	include <wininet.h>
#endif

DataReader::DataReader(std::list<InstallDataBlock *> &installItems, RebootAction &rebootAction, MyWindow const *parent)
	: QThread(),
      _installItems(installItems),
      _rebootAction(rebootAction),
      _parent(parent)
{
}

DataReader::~DataReader()
{
}

void DataReader::run()
{
    InstallDataHandler *handler = new InstallDataHandler(_installItems, _rebootAction, _parent);

    QXmlSimpleReader structReader;
    structReader.setContentHandler(handler);
    structReader.setErrorHandler(handler);

    QFile structFile("genInstall.xml");
    if(!structFile.exists())
        return;

    QXmlInputSource source(&structFile);
    structReader.parse(&source);

    for(std::list<InstallDataBlock *>::iterator bit = _installItems.begin(); bit != _installItems.end(); ++ bit) {
        for(std::list<InstallData *>::iterator it = (*bit)->getBlock()->begin();
            it != (*bit)->getBlock()->end();
            ++ it) {

#ifdef Q_OS_WIN
            QString fileToInstall = "packages\\" + (*it)->getDirectory() + '\\' + (*it)->getPkg();
#else
            QString fileToInstall = "packages/" + (*it)->getDirectory() + '/' + (*it)->getPkg();
#endif
            if(access(fileToInstall.toStdString().c_str(), R_OK) != 0) {
#ifndef NDEBUG
                std::cerr << "Package does not exist: " << fileToInstall.toStdString() << std::endl;
#endif
                delete *it;
                (*bit)->getBlock()->erase(it);
                -- it;
                continue;
            }

            if(!(*it)->getDirFolder().isEmpty()) {
#ifndef NDEBUG
                std::cerr << "Check directory folder: " << (*it)->getDirFolder().toStdString() << std::endl;
#endif
                QString dir = (*it)->getDirFolder();
                char const *sysroot = getenv("SYSTEMROOT");
                if(sysroot == 0)
                    sysroot = "C:\\Windows";
                dir = dir.replace("%SYSTEMROOT%", sysroot);
                QDir dirReader(dir);
                if(dirReader.exists()) {
                    QStringList filters;
                    filters << (*it)->getDirMatches();
                    dirReader.setNameFilters(filters);
                    if(!dirReader.entryInfoList(QDir::Dirs).empty())
                        (*it)->setInstalled();
#ifndef NDEBUG
                    else
                        std::cerr << "no match of " << (*it)->getDirMatches().toStdString() << " in " << dir.toStdString() << std::endl;
#endif
                }
#ifndef NDEBUG
                else
                    std::cerr << "no directory " << dir.toStdString() << std::endl;
#endif
            }

            if(!(*it)->getRegFolder().isEmpty()) {
#ifdef Q_OS_WIN
                HKEY base;
                QString baseKey;
                if((*it)->getRegFolder().left(18) == "HKEY_LOCAL_MACHINE") {
                    base = HKEY_LOCAL_MACHINE;
                    baseKey = (*it)->getRegFolder().mid(19);
                }
                else if((*it)->getRegFolder().left(17) == "HKEY_CLASSES_ROOT") {
                    base = HKEY_CLASSES_ROOT;
                    baseKey = (*it)->getRegFolder().mid(18);
                }
                else if((*it)->getRegFolder().left(17) == "HKEY_CURRENT_USER") {
                    base = HKEY_CURRENT_USER;
                    baseKey = (*it)->getRegFolder().mid(18);
                }
                else if((*it)->getRegFolder().left(10) == "HKEY_USERS") {
                    base = HKEY_USERS;
                    baseKey = (*it)->getRegFolder().mid(11);
                }
                else if((*it)->getRegFolder().left(19) == "HKEY_CURRENT_CONFIG") {
                    base = HKEY_CURRENT_CONFIG;
                    baseKey = (*it)->getRegFolder().mid(20);
                }
                else {
                    base = 0;
#ifndef NDEBUG
                    std::cerr << "Did not understand: " << (*it)->getRegFolder().toStdString() << std::endl;
#endif
                }

                if(base != 0) {
#ifndef NDEBUG
                    std::cerr << "Checking: " << (*it)->getRegFolder().toStdString() << std::endl;
                    std::cerr << "BaseKey: " << baseKey.toStdString() << std::endl;
#endif
                    HKEY reg_key;
                    LONG error = RegOpenKeyEx(base, baseKey.toAscii(), 0, KEY_READ|KEY_QUERY_VALUE, &reg_key);
                
                    if(error == ERROR_SUCCESS) {
#ifndef NDEBUG
                        std::cerr << "Key dir found: " << (*it)->getRegFolder().toStdString() << std::endl;
#endif
                        if((*it)->getRegExistenceOnly()) {
                            (*it)->setInstalled();
                        }
                        else if(!(*it)->getRegKey().isEmpty() || !(*it)->getRegStartsWith().isEmpty()) {
                            DWORD type = REG_SZ;
                            DWORD size = 1024;
                            char key[size + 1];
                            *key = 0;
                    
                            RegQueryValueEx(reg_key, (*it)->getRegKey().toAscii(), NULL, &type, (BYTE *)&key, &size);
                            if(!(*it)->getRegKey().isEmpty() && (*it)->getStringValue() == key)
                                (*it)->setInstalled();
                            else if(!(*it)->getRegStartsWith().isEmpty()) {
                                char keyStart[(*it)->getRegStartsWith().length() + 1];
                                memcpy(keyStart, key, (*it)->getRegStartsWith().length());
                                keyStart[(*it)->getRegStartsWith().length()] = 0;
                                if((*it)->getRegStartsWith() == keyStart)
                                    (*it)->setInstalled();
#ifndef NDEBUG
                                else
                                    std::cerr << "key does not match: check for '" << (*it)->getStringValue().toStdString()
                                              << "' found '" << key << "'" << std::endl;
#endif
                            }
#ifndef NDEBUG
                            else
                                std::cerr << "key does not match: check for '" << (*it)->getStringValue().toStdString()
                                          << "' found '" << key << "'" << std::endl;
#endif
                        }
#ifndef NDEBUG
                        else
                            std::cerr << "Check for key, but no key is here: " << (*it)->getRegFolder().toStdString() << std::endl;
#endif
                        RegCloseKey(reg_key);
                    }
#ifndef NDEBUG
                    else
                        std::cerr << "No key dir: " << (*it)->getRegFolder().toStdString() << std::endl;
#endif
                }
#endif
            }
        }

        if((*bit)->getBlock()->empty()) {
#ifndef NDEBUG
            std::cerr << "Leerer Install Block of type " << (*bit)->getAppliesString() << " entfernt " << std::endl;
#endif
            delete *bit;
            _installItems.erase(bit);
            -- bit;
        }
    }
}

#include "DataReader.moc"
