// genericInstaller -- update windows and office
//
// Copyright (C) 2007,2008
// Wolfgang Formann
//
// this file is part of the programm genericInstaller.  This program is
// free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your
// option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "DecipherProcess.h"

#include "MyWindow.h"
#include "InstallPage.h"
#include "InstallProcess.h"
#include "InstallData.h"
#include "InstallationStatus.h"

#include <iostream>
#include <fcntl.h>

DecipherProcess::DecipherProcess(InstallPage *parent, std::list<InstallData *>::iterator it)
	: QProcess(parent),
      _it(it),
      _parent(parent)
{
}

DecipherProcess::~DecipherProcess()
{
}

void DecipherProcess::run()
{
    QString hint = "Installiert wird: KB";
    hint += (*_it)->getKb() + " &ndash; <b>" + (*_it)->getShort() + "</b>";
    _parent->setLongDescription(hint);
    (*_it)->makeVisible();
    (*_it)->setInstall();

    // openssl enc -bf -pass pass:a -e -in hello -out hello.cipher
#ifdef Q_OS_WIN
    QString source = "packages\\" + (*_it)->getDirectory() + '\\' + (*_it)->getPkg();
    _decryptedFile = InstallationStatus::getInstance(_parent)->getTempDir() + '\\' + (*_it)->getPkg();
#else
    QString source = "packages/" + (*_it)->getDirectory() + '/' + (*_it)->getPkg();
    _decryptedFile = InstallationStatus::getInstance(_parent)->getTempDir() + '/' + (*_it)->getPkg();
#endif

    QString command;
    QStringList args;
    command = "openssl";
    args << "enc";
    args << "-bf";
    args << "-pass";
    args << "pass:" + tr("Nobba") + tr("rpy") + tr("ok0");
    args << "-d";
    args << "-in";
    args << source;
    args << "-out";
    args << _decryptedFile;

    setProcessChannelMode(QProcess::ForwardedChannels);
    connect(this, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(didDecipher(int, QProcess::ExitStatus)));
    start(command, args);
    if(!waitForStarted(6000)) {
#ifndef NDEBUG
        std::cerr << "failed to start '" << command.toStdString() << "'" << std::endl;
#endif
        (*_it)->setDone();
#ifndef Q_OS_WIN
        sleep(3);
#endif
        ::unlink(_decryptedFile.toStdString().c_str());
        _parent->install(0, ++ _it);
        delete this;
    }
}

void DecipherProcess::didDecipher(int exitCode, QProcess::ExitStatus exitStatus)
{
    std::cerr << "didDecipher() " << exitCode << " " << exitStatus << std::endl;
    if(access(_decryptedFile.toStdString().c_str(), F_OK) == 0) {
        if(exitCode == 0) {
            InstallProcess *p = new InstallProcess(_parent, _it, true);
#ifndef NDEBUG
            std::cerr << "did decipher '" << _decryptedFile.toStdString() << "'" << std::endl;
#endif
#ifndef Q_OS_WIN
            chmod(_decryptedFile.toStdString().c_str(), 0777);
#endif
            p->run();
        }
        else {
            ::unlink(_decryptedFile.toStdString().c_str());
            _parent->install(0, ++ _it);
        }
    }
    else {
#ifndef NDEBUG
        std::cerr << "failed to decipher '" << _decryptedFile.toStdString() << "'" << std::endl;
#endif
        (*_it)->setDone();
        _parent->install(0, ++ _it);
    }
    delete this;
}

#include "DecipherProcess.moc"
