// genericInstaller -- update windows and office
//
// Copyright (C) 2008
// Wolfgang Formann
//
// this file is part of the programm genericInstaller.  This program is
// free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your
// option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <assert.h>
#ifndef NDEBUG
#include <iostream>
#endif

#include "InstallData.h"
#include "WindowsType.h"
#include "MyWindow.h"

InstallDataBlock::InstallDataBlock()
    : _isSkipable(false)
{
}

InstallData::InstallData()
	: _regExistenceOnly(false),
      _position(-1),
      _minimize(0),
      _installTime(6000),
      _ignoreAfter(0),
	  _isInstalled(false),
	  _isImportant(false),
//      _ifIe7(false),
//      _ifNotIe7(false),
      _netNeeded(false),
      _isDefault(false),
      _isCheckable(true),
      _isSilent(false),
      _isEncrypted(false),
      _execType(eTypeWusa),
      _allowDeselect(true),
      _entry(0)
{
}

void InstallData::setPrimaryPkg(const QString &pkg)
{
    for(std::list<QString>::iterator it = _pkg.begin(); it != _pkg.end(); ++ it) {
        if(*it == pkg) {
            if(it == _pkg.begin())
                break;
            _pkg.erase(it);
            _pkg.push_front(pkg);
            break;
        }
    }
}

const InstallDataHandler::stateDescription InstallDataHandler::_description[] = {
    { "allpatches", eNone,       eAllPAtches,        false, false, false },
    { "reboot",      eAllPAtches, eReboot,           false, false, true  },
    { "os",           eReboot,     eOs,              false, false, false },
    { "id",            eOs,         eId,             false, false, false },
    { "command",       eOs,         eCommand,        false, false, false },
    { "param",         eOs,         eParam,          false, false, false },
    { "patchgroup",  eAllPAtches, ePatchGroup,       false, true,  false },
    { "applies",      ePatchGroup, eApplies,         false, false, false },
    { "message",      ePatchGroup, eMessage,         false, false, false },
    { "endmessage",   ePatchGroup, eEndMessage,      false, false, false },
    { "bootmessage",  ePatchGroup, eBootMessage,     false, false, false },
    { "header",       ePatchGroup, eHeader,          false, false, false },
    { "skipable",     ePatchGroup, eSkipable,        false, false, false },
    { "update",       ePatchGroup, eUpdate,          true,  false, false },
    { "desc",          eUpdate,     eDesc,           false, false, false }, //
    { "pkg",           eUpdate,     ePkg,            false, false, false }, //
    { "encrypted",     eUpdate,     eEncrypted,      false, false, false }, //
    { "info",          eUpdate,     eInfo,           false, false, false },
    { "position",      eUpdate,     ePosition,       false, false, false },
    { "default",       eUpdate,     eDefault,        false, false, false }, //
    { "checkable",     eUpdate,     eCheckable,      false, false, false },
    { "silent",        eUpdate,     eSilent,         false, false, false },
    { "installtime",   eUpdate,     eInstallTime,    false, false, false },
    { "ignoreafter",   eUpdate,     eIgnoreAfter,    false, false, false },
    { "directory",     eUpdate,     eDirectory,      false, false, false },
    { "dirmatches",    eDirectory,  eDirMatches,     false, false, false },
    { "registry",      eUpdate,     eRegistry,       false, false, false },
    { "folder",         eRegistry,   eFolder,        false, false, false },
    { "existenceonly",  eRegistry,   eExistenceOnly, false, false, false },
    { "startswith",     eRegistry,   eStartsWith,    false, false, false },
    { "key",            eRegistry,   eKey,           false, false, false },
    { "stringvalue",    eRegistry,   eStringValue,   false, false, false },
    { "closeWindow",   eUpdate,     eCloseWindow,    false, false, false },
    { "important",     eUpdate,     eImportant,      false, false, false }, //
    { "network",       eUpdate,     eNetNeeded,      false, false, false },
    { "short",         eUpdate,     eShort,          false, false, false }, //
    { "append",        eUpdate,     eAddDesc,        false, false, false },
    { "type",          eUpdate,     eType,           false, false, false },
    { "switches",      eUpdate,     eSwitches,       false, false, false }, //
    { "postswitches",  eUpdate,     ePostSwitches,   false, false, false }, //
    { "minimize",      eUpdate,     eMinimize,       false, false, false },
    { "dir",           eUpdate,     eDir,            false, false, false },
    { "kb",            eUpdate,     eKb,             false, false, false },
};

InstallDataHandler::InstallDataHandler(std::list<InstallDataBlock *> &blocks, RebootAction &rebootAction, MyWindow const *parent)
	: _blocks(blocks),
      _rebootAction(rebootAction),
      _isOsOk(false),
      _actualBlock(0),
      _actual(0),
      _state(eNone),
      _parent(parent)
{
}

bool InstallDataHandler::fatalError(const QXmlParseException &exception)
{
#ifndef NDEBUG
    std::cerr << "Fatal error on line" << exception.lineNumber()
               << ", column" << exception.columnNumber() << ":"
               << exception.message().toStdString() << std::endl;;
#endif

    return false;
}

bool InstallDataHandler::error(const QXmlParseException &exception)
{
#ifndef NDEBUG
    std::cerr << "Error on line" << exception.lineNumber()
               << ", column" << exception.columnNumber() << ":"
               << exception.message().toStdString() << std::endl;;
#endif

    return false;
}

bool InstallDataHandler::warning(const QXmlParseException &exception)
{
#ifndef NDEBUG
    std::cerr << "Warning on line" << exception.lineNumber()
               << ", column" << exception.columnNumber() << ":"
               << exception.message().toStdString() << std::endl;;
#endif

    return false;
}

bool InstallDataHandler::startElement(const QString &nameSpace, const QString &lName, const QString &qName, const QXmlAttributes &attr)
{
//#ifndef NDEBUG
//	std::cerr << "Read Element nspc:" << nameSpace.toStdString()
//	          << " lnm: " << lName.toStdString()
//	          << " qnm: " << qName.toStdString()
//	          << " #at: " << attr.length()
//	          << std::endl;
//#endif
//	if(attr.length() == 0) {
//		std::cerr << "Element nspc:" << nameSpace.toStdString()
//		          << " lnm: " << lName.toStdString()
//		          << " qnm: " << qName.toStdString()
//		          << " #at: " << attr.length()
//		          << std::endl;
//		return true;
//	}

    for(unsigned int i = 0; i < sizeof(_description) / sizeof(*_description); ++ i) {
        if(lName == _description[i]._name) {
            if(_state != _description[i]._inState) {
#ifndef NDEBUG
                std::cerr << "Element nspc:" << nameSpace.toStdString()
                          << " lnm: " << lName.toStdString()
                          << " qnm: " << qName.toStdString()
                          << " #at: " << attr.length()
                          << std::endl;
#endif
            }
            else {
                if(_description[i]._makeReboot) {
                    assert(_actualBlock == 0);
                    assert(_actual == 0);
                    _actualBlock = new InstallDataBlock();
                }
                else if(_description[i]._makeBlock) {
                    assert(_actualBlock == 0);
                    assert(_actual == 0);
                    _actualBlock = new InstallDataBlock();
                }
                else if(_description[i]._makeNew) {
                    assert(_actual == 0);
                    _actual = new InstallData();
                }
                _state = _description[i]._outState;
                for(int j = 0; j < attr.length(); ++ j) {
//#ifndef NDEBUG
//                    std::cerr << "Read Attribute :" << j
//                              << " lnm: " << attr.localName(j).toStdString()
//                              << " typ: " << attr.type(j).toStdString()
//                              << " uri: " << attr.uri(j).toStdString()
//                              << " val: " << attr.value(j).toStdString()
//                              << std::endl;
//#endif
                    for(unsigned int i = 0; i < sizeof(_description) / sizeof(*_description); ++ i) {
                        if(attr.localName(j) == _description[i]._name) {
                            if(_state != _description[i]._inState) {
#ifndef NDEBUG
                                std::cerr << "Attribute :" << j
                                          << " lnm: " << attr.localName(j).toStdString()
                                          << " typ: " << attr.type(j).toStdString()
                                          << " uri: " << attr.uri(j).toStdString()
                                          << " val: " << attr.value(j).toStdString()
                                          << std::endl;
#endif
                            }
                            else {
                                if(_description[i]._makeReboot) {
                                    assert(0);
                                }
                                else if(_description[i]._makeBlock) {
                                    assert(0);
                                }
                                else if(_description[i]._makeNew) {
                                    assert(0);
                                }
                                _state = _description[i]._outState;
                                characters(attr.value(j));
                                _state = _description[i]._inState;
                            }
                        }
                    }
                }
                return true;
            }
        }
    }

#ifndef NDEBUG
    std::cerr << "Not matching Element nspc:" << nameSpace.toStdString()
              << " lnm: " << lName.toStdString()
              << " qnm: " << qName.toStdString()
              << " #at: " << attr.length()
              << std::endl;
#endif

	return true;
}

bool InstallDataHandler::endElement(const QString &nameSpace, const QString &lName, const QString &qName)
{
    for(unsigned int i = 0; i < sizeof(_description) / sizeof(*_description); ++ i) {
        if(lName == _description[i]._name) {
            if(_state != _description[i]._outState) {
#ifndef NDEBUG
                std::cerr << "Element nspc:" << nameSpace.toStdString()
                          << " lnm: " << lName.toStdString()
                          << " qnm: " << qName.toStdString()
                          << std::endl;
#endif
            }
            else {
                if(_description[i]._makeReboot) {
                    assert(_actual == 0);
                    assert(_actualBlock != 0);
                    _isOsOk = false;
                }
                else if(_description[i]._makeBlock) {
                    assert(_actual == 0);
                     _blocks.push_back(_actualBlock);
                     _actualBlock = 0;
                }
                else if(_description[i]._makeNew) {
                    assert(_actualBlock != 0);
                    if(_actual->getPosition() == -1)
                        _actualBlock->push_back(_actual);
                    else
                        _actualBlock->push_front(_actual);
                    _actual = 0;
                }
                _state = _description[i]._inState;
                return true;
            }
        }
    }

#ifndef NDEBUG
    std::cerr << "Not matching Element nspc:" << nameSpace.toStdString()
              << " lnm: " << lName.toStdString()
              << " qnm: " << qName.toStdString()
              << std::endl;
#endif
    return true;
}

bool InstallDataHandler::characters(const QString &ch)
{
    QString tch = ch.trimmed();
    for(int pos = 0; (pos = tch.indexOf("\r\n", pos)) > 0; )
        tch = tch.replace(pos, 2, " ");
    for(int pos = 0; (pos = tch.indexOf("\n", pos)) > 0; )
        tch = tch.replace(pos, 1, " ");
    for(int pos = 0; (pos = tch.indexOf("  ", pos)) > 0; )
        tch = tch.replace(pos, 2, " ");

    switch(_state) {
    case eDesc:
        _actual->setDesc(tch);
        break;

    case ePkg:
        _actual->setPkg(tch);
        break;

    case eEncrypted:
        _actual->setEncrypted(tch);
        break;

    case eInfo:
        _actual->setInfo(tch);
        break;

    case ePosition:
        _actual->setPosition(tch);
        _actual->setNoDeselect();
        break;

    case eDefault:
        _actual->setDefault(tch);
        break;

    case eCheckable:
        _actual->setCheckable(tch);
        break;

    case eSilent:
        _actual->setSilent(tch);
        break;

    case eInstallTime:
        _actual->setInstallTime(tch);
        break;

    case eIgnoreAfter:
        _actual->setIgnoreAfter(tch);
        break;

    case eImportant:
        _actual->setImportant(tch);
        break;

    case eNetNeeded:
        _actual->setNetNeeded(tch);
        break;

    case eShort:
        _actual->setShort(tch);
        break;

    case eAddDesc:
        _actual->setAddDesc(tch);
        break;

    case eType:
        _actual->setExecType(tch);
        break;

    case ePostSwitches:
        _actual->setPostSwitches(tch);
        break;

    case eSwitches:
        _actual->setSwitches(tch);
        break;

    case eFolder:
        _actual->setRegFolder(tch);
        break;

    case eCloseWindow:
        _actual->setWindowsClassToClose(tch);
        break;

    case eExistenceOnly:
        _actual->setRegExistenceOnly(tch);
        break;

    case eKey:
        _actual->setRegKey(tch);
        break;

    case eStringValue:
        _actual->setRegStringValue(tch);
        break;

    case eStartsWith:
        _actual->setRegStartsWith(tch);
        break;

    case eDirectory:
        _actual->setDirFolder(tch);
        break;

    case eDirMatches:
        _actual->setDirMatches(tch);
        break;

    case eMinimize:
        _actual->setMinimize(tch);
        break;

    case eDir:
        _actual->setDirectory(tch);
        break;

    case eKb:
        _actual->setKb(tch);
        break;

    case eEndMessage:
        _actualBlock->setEndMessage(tch);
        break;

    case eBootMessage:
        _actualBlock->setBootMessage(tch);
        break;

    case eMessage:
        _actualBlock->setMessage(tch);
        break;

    case eHeader:
        _actualBlock->setHeader(tch);
        break;

    case eApplies:
        _actualBlock->setApplies(tch);
        break;

    case eSkipable:
        _actualBlock->setSkipable(tch);
        break;

    case eId:
        if(tch == "vista" && _parent->getWindowsType() == WindowsVista)
            _isOsOk = true;
        else if(tch == "xp" && _parent->getWindowsType() == WindowsXP)
            _isOsOk = true;
        else
            _isOsOk = false;
        break;

    case eCommand:
        if(_isOsOk)
            _rebootAction.setCommand(tch);
        break;

    case eParam:
        if(_isOsOk)
            _rebootAction.setArguments(tch);
        break;

    case eOs:
    case eReboot:
    case eRegistry:
    case eAllPAtches:
    case eUpdate:
    case eNone:
    default:
#ifndef NDEBUG
        if(!tch.isEmpty())
            std::cerr << "tch: " + tch.toStdString() << std::endl;
#endif
        ;
    }

    return true;
}
