// genericInstaller -- update windows and office
//
// Copyright (C) 2008
// Wolfgang Formann
//
// this file is part of the programm genericInstaller.  This program is
// free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your
// option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef InstallData_h
#define InstallData_h

#include <list>
#ifndef NDEBUG
#include <iostream>
#endif
#include <Qt/qstring.h>
#include <Qt/qxml.h>

#include "InstallItem.h"

enum applyValue {
    eNone,
    eXpForce,
    eVistaForce,
    eXpSp3,
    eXpIe7,
    // Knopf "berspringen"
    eXpWmp11,
    eVistaPreSp1,
    eVistaSp1,
    eOffice2003Sp3,
    eOffice2007Sp1,
    eNet,
    eXp,
    eVista,
    eOffice2003,
    eOffice2007
};

class page2;
class InstallData;
class MyWindow;

class RebootAction {
public:
	void setCommand(const QString &command) {
#ifndef NDEBUG
        std::cerr << "Setting reboot Command: " << command.toStdString() << std::endl;
#endif
		_command = command;
	}
	const QString &getCommand() const {
		return _command;
	}

	void setArguments(const QString &arguments) {
		_arguments = arguments;
	}
	const QString &getArguments() const {
		return _arguments;
	}

private:
    QString _command;
    QString _arguments;
};

class InstallDataBlock {
public:
	InstallDataBlock();

    void setApplies(const QString &applies) {
        if(applies == "xpforce")
            _applies = eXpForce;
        else if(applies == "vistaforce")
            _applies = eVistaForce;
        else if(applies == "xpsp3")
            _applies = eXpSp3;
        else if(applies == "xpie7")
            _applies = eXpIe7;
        else if(applies == "xpwmp11")
            _applies = eXpWmp11;
        else if(applies == "vistapresp1")
            _applies = eVistaPreSp1;
        else if(applies == "vistasp1")
            _applies = eVistaSp1;
        else if(applies == "o2k3sp3")
            _applies = eOffice2003Sp3;
        else if(applies == "o2k7sp1")
            _applies = eOffice2007Sp1;
        else if(applies == "net")
            _applies = eNet;
        else if(applies == "xp")
            _applies = eXp;
        else if(applies == "vista")
            _applies = eVista;
        else if(applies == "o2k3")
            _applies = eOffice2003;
        else if(applies == "o2k7")
            _applies = eOffice2007;
        else {
#ifndef NDEBUG
            std::cerr << "Unknown applies-Value:" << applies.toStdString() << std::endl;
#endif
            _applies = eNone;
        }
	}
	applyValue getApplies() const {
		return _applies;
	}
#ifndef NDEBUG
    void setApplies(applyValue applies) {
        _applies = applies;
    }
	char const *getAppliesString() const {
        switch(_applies) {
        case eNone:
            return "None";
        case eXpForce:
            return "XpForce";
        case eVistaForce:
            return "VistaForce";
        case eXpSp3:
            return "XpSp3";
        case eXpIe7:
            return "XpIe7";
        case eXpWmp11:
            return "XpWmp11";
        case eVistaPreSp1:
            return "VistaPreSp1";
        case eVistaSp1:
            return "VistaSp1";
        case eOffice2003Sp3:
            return "Office2003Sp3";
        case eOffice2007Sp1:
            return "Office2007Sp1";
        case eNet:
            return "Net";
        case eXp:
            return "Xp";
        case eVista:
            return "Vista";
        case eOffice2003:
            return "Office2003";
        case eOffice2007:
            return "Office2007";
        }
        return "Unbekannt";
	}
#endif

	void setMessage(const QString &message) {
		_message = message;
	}
	const QString &getMessage() const {
		return _message;
	}

	void setEndMessage(const QString &endMessage) {
		_endMessage = endMessage;
	}
	const QString &getEndMessage() const {
		return _endMessage;
	}

	void setBootMessage(const QString &bootMessage) {
		_bootMessage = bootMessage;
	}
	const QString &getBootMessage() const {
		return _bootMessage;
	}

    void setHeader(const QString &header) {
		_header = header;
	}
	const QString &getHeader() const {
		return _header;
	}

    void setSkipable(const QString &isSkipable) {
#ifndef NDEBUG
        std::cerr << "set skipable to: " << isSkipable.toStdString() << " / " << isSkipable.toInt() << std::endl;
#endif
		_isSkipable = isSkipable.toInt() != 0;
	}
    void setSkipable() {
		_isSkipable = true;
	}
	bool getSkipable() const {
		return _isSkipable;
	}

    std::list<InstallData *> *getBlock() {
		return &_actualBlock;
	}

    void push_back(InstallData *actual) {
        _actualBlock.push_back(actual);
    }
    void push_front(InstallData *actual) {
        _actualBlock.push_front(actual);
    }

private:
	applyValue _applies;
	QString _message;
	QString _header;
	QString _endMessage;
    QString _bootMessage;
    bool _isSkipable;
    std::list<InstallData *> _actualBlock;
};

class InstallData {
public:
	InstallData();
//	~InstallData();
	void setDesc(const QString &desc) {
		_desc = desc;
	}
	const QString &getDesc() const {
		return _desc;
	}

	void setAddDesc(const QString &addDesc) {
		_addDesc = addDesc;
	}
	const QString &getAddDesc() const {
		return _addDesc;
	}

	void setShort(const QString &shortDesc) {
		_short = shortDesc;
	}
	const QString &getShort() const {
		return _short;
	}

    enum Status {
        eInstalled,
        eImportant,
        eNeedNet,
        eNormal
    };

    Status getStatus() const {
        if(isInstalled())
            return eInstalled;
        if(isImportant())
            return eImportant;
        if(needNet())
            return eNeedNet;
        return eNormal;
	}

    void setPkg(const QString &pkg) {
		_pkg.push_back(pkg);
	}
    void setPrimaryPkg(const QString &pkg);
	const QString &getPkg() const {
		return _pkg.front();
	}

	void setInfo(const QString &info) {
		_info = info;
	}
	const QString &getInfo() const {
		return _info;
	}

	void setSwitches(const QString &switches) {
		_switches = switches;
	}
	const QString &getSwitches() const {
		return _switches;
	}

	void setPostSwitches(const QString &postSwitches) {
		_postSwitches = postSwitches;
	}
	const QString &getPostSwitches() const {
		return _postSwitches;
	}

	void setEntry(InstallPage *mainWin) {
        _entry = new InstallItem(mainWin, this);
	}
	void deleteEntry() {
        if(_entry) {
            _entry->deleteLater();
            _entry = 0;
        }
	}
	InstallItem *getEntry() {
		return _entry;
	}

    void setDone() {
        _entry->setDone();
    }

    void setInstall() {
        _entry->setInstall();
    }

    void resetInstall() {
        _entry->resetInstall(this);
    }

    void setInstalledIcon() {
        _entry->setInstalledIcon();
    }

    void makeVisible() {
        _entry->makeVisible();
    }

    void disableCheckbox() {
        _entry->disableCheckbox();
    }

    void setInstalled() {
		_isInstalled = true;
	}
	bool isInstalled() const {
		return _isInstalled;
	}

    void setDefault(const QString &isDefault) {
		_isDefault = isDefault.toInt() != 0;
	}
    void setDefault() {
		_isDefault = true;
	}
	bool isDefault() const {
		return _isDefault;
	}

    void setEncrypted(const QString &isEncrypted) {
		_isEncrypted = isEncrypted.toInt() != 0;
	}
    void setEncrypted() {
		_isEncrypted = true;
	}
	bool isEncrypted() const {
		return _isEncrypted;
	}

    void setCheckable(const QString &isCheckable) {
		_isCheckable = isCheckable.toInt() != 0;
	}
    void setCheckable() {
		_isCheckable = true;
	}
	bool isCheckable() const {
		return _isCheckable;
	}

    void setInstallTime(const QString &installTime) {
		_installTime = installTime.toInt();
	}
	int getInstallTime() const {
		return _installTime;
	}

    void setIgnoreAfter(const QString &ignoreAfter) {
		_ignoreAfter = ignoreAfter.toInt();
	}
	int getIgnoreAfter() const {
		return _ignoreAfter;
	}

    void setSilent(const QString &isSilent) {
		_isSilent = isSilent.toInt() != 0;
	}
    void setSilent() {
		_isSilent = true;
	}
	bool isSilent() const {
		return _isSilent;
	}

    enum PackageType {
        eTypeExe,
        eTypeMsi,
        eTypeWusa,
        eFirefox
    };

    void setExecType(const QString &type) {
		_execType = (type == "exe" ? eTypeExe : type == "msi" ? eTypeMsi : type == "firefox" ? eFirefox : eTypeWusa);
	}
	PackageType _getExecType() const {
		return _execType;
	}

    void setImportant(const QString &important) {
		_isImportant = important.toInt() != 0;
	}
    void setImportant() {
		_isImportant = true;
	}
	bool isImportant() const {
		return _isImportant;
	}

    void setNetNeeded(const QString &netNeeded) {
		_netNeeded = netNeeded.toInt() != 0;
	}
    void setNetNeeded() {
		_netNeeded = true;
	}
	bool needNet() const {
		return _netNeeded;
	}

    void setRegExistenceOnly(const QString &regExistenceOnly) {
		_regExistenceOnly = regExistenceOnly.toInt() != 0;
	}
    void setRegExistenceOnly() {
		_regExistenceOnly = true;
	}
	bool getRegExistenceOnly() const {
		return _regExistenceOnly;
	}

    void setMinimize(const QString &minimize) {
		_minimize = minimize.toInt();
#ifndef NDEBUG
        std::cerr << "minimize: " << minimize.toStdString() << " set to " << _minimize << std::endl;
#endif
	}
    void setMinimize() {
		_minimize = 1;
	}
	int getMinimize() const {
		return _minimize;
	}

    void setDirectory(const QString &directory) {
		_directory = directory;
	}
    void setDirectory() {
		_directory = "";
	}
	QString const &getDirectory() const {
		return _directory;
	}

    void setKb(const QString &kb) {
		_kb = kb;
	}
    void setKb() {
		_kb = "";
	}
	QString const &getKb() const {
		return _kb;
	}

    void setRegFolder(const QString &regFolder) {
		_regFolder = regFolder;
	}
	QString const &getRegFolder() const {
		return _regFolder;
	}

    void setDirFolder(const QString &dirFolder) {
		_dirFolder = dirFolder;
	}
	QString const &getDirFolder() const {
		return _dirFolder;
	}

    void setDirMatches(const QString &dirStartsWith) {
		dirMatches = dirStartsWith;
	}
	QString const &getDirMatches() const {
		return dirMatches;
	}

    void setWindowsClassToClose(const QString &windowsClassToClose) {
		_windowsClassToClose = windowsClassToClose;
	}
	QString const &getWindowsClassToClose() const {
		return _windowsClassToClose;
	}

    void setRegKey(const QString &regKey) {
		_regKey = regKey;
	}
	QString const &getRegKey() const {
		return _regKey;
	}

    void setRegStartsWith(const QString &regKeyStartsWith) {
		_regKeyStartsWith = regKeyStartsWith;
	}
	QString const &getRegStartsWith() const {
		return _regKeyStartsWith;
	}

    void setRegStringValue(const QString &regStringValue) {
		_regStringValue = regStringValue;
	}
	QString const &getStringValue() const {
		return _regStringValue;
	}

    void setNoDeselect() {
		_allowDeselect = false;
	}
	bool isDeselect() const {
		return _allowDeselect;
	}

    void setPosition(const QString &position) {
		_position = position.toInt();
	}
	int getPosition() const {
		return _position;
	}

private:
	QString _info;
	QString _desc;
    QString _addDesc;
	QString _short;
    QString _regFolder;
    QString _windowsClassToClose;
    bool _regExistenceOnly;
    QString _regKey;
    QString _regKeyStartsWith;
    QString _regStringValue;
    QString _dirFolder;
    QString dirMatches;
	std::list<QString> _pkg;
	QString _switches;
    QString _postSwitches;
	int _position;
    int _minimize;
    QString _directory;
    QString _kb;
    int _installTime;
    int _ignoreAfter;
    bool _isInstalled;
    bool _isImportant;
//    bool _ifIe7;
//    bool _ifNotIe7;
    bool _netNeeded;
    bool _isDefault;
    bool _isCheckable;
    bool _isSilent;
    bool _isEncrypted;
    PackageType _execType;
    bool _allowDeselect;
    InstallItem *_entry;
};

class InstallDataHandler : public QXmlDefaultHandler
{
public:
	InstallDataHandler(std::list<InstallDataBlock *> &blocks, RebootAction &rebootAction, MyWindow const *parent);
	bool fatalError(const QXmlParseException &exception);
	bool error(const QXmlParseException &exception);
	bool warning(const QXmlParseException &exception);
	bool characters(const QString &ch);
	bool startElement(const QString &nameSpace, const QString &lName, const QString &qName, const QXmlAttributes &attr);
	bool endElement(const QString &nameSpace, const QString &lName, const QString &qName);

private:
    std::list<InstallDataBlock *> &_blocks;
    RebootAction &_rebootAction;
    bool _isOsOk;
    InstallDataBlock *_actualBlock;
    InstallData *_actual;
    enum State {
        eAllPAtches,
        ePatchGroup,
        eUpdate,
        ePkg,
        eShort,
        eAddDesc,
        eDesc,
        eEncrypted,
        eInfo,
        ePosition,
        eSilent,
        eDefault,
        eCheckable,
        eImportant,
        eNetNeeded,
        eType,
        eSwitches,
        ePostSwitches,
        eInstallTime,
        eIgnoreAfter,
        eDirectory,
        eDirMatches,
        eRegistry,
        eFolder,
        eCloseWindow,
        eExistenceOnly,
        eKey,
        eStringValue,
        eStartsWith,
        eMinimize,
        eDir,
        eKb,
        eApplies,
        eSkipable,
        eMessage,
        eEndMessage,
        eBootMessage,
        eHeader,
        eReboot,
        eOs,
        eId,
        eCommand,
        eParam,
        eNone
    };
    State _state;
    MyWindow const *_parent;
    struct stateDescription {
        char const *_name;
        enum State _inState;
        enum State _outState;
        bool _makeNew;
        bool _makeBlock;
        bool _makeReboot;
    };
    static const stateDescription _description[];
};
#endif
