// genericInstaller -- update windows and office
//
// Copyright (C) 2008
// Wolfgang Formann
//
// this file is part of the programm genericInstaller.  This program is
// free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your
// option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef NDEBUG
#include <iostream>
#endif

#include "InstallItem.h"
#include "InstallData.h"
#include "InstallPage.h"
#include "MyWindow.h"

#ifdef Q_OS_WIN
#	include <windows.h>
#endif

#include <Qt/qcoreevent.h>
#include <Qt/qmovie.h>

InstallItem::InstallItem(InstallPage *mainWin, InstallData const *data)
	: QHBoxLayout(),
      _checkbox(0),
      _status(new MyLabel(this)),
      _mainWin(mainWin),
      _data(data)
{
    resetInstall(data);
    switch(data->getStatus()) {
    case InstallData::eInstalled:
        _status->setPixmap(QPixmap(":/images/okay"));
        break;
    case InstallData::eImportant:
        _status->setPixmap(QPixmap(":/images/important"));
        break;
    case InstallData::eNormal:
//        break;
        ;
    }
    _status->setMinimumSize(22, 22);
    addWidget(_status, 0);

    _checkbox = new MyCheckBox(mainWin, data, (data->isDefault() && data->isInstalled() == false) ? Qt::Checked : Qt::Unchecked, !data->isCheckable());
    _checkbox->setText(data->getKb().isEmpty() ? "Ohne KB-Nummer" : ("KB" + data->getKb()));
    QFont f(_checkbox->font());
    f.setBold(true);
    _checkbox->setFont(f);

    connect(_checkbox, SIGNAL(stateChanged(int)), mainWin, SLOT(doSelectExtension(int)));
    addWidget(_checkbox, 0);

    if(true || data->getKb().isEmpty()) {
        MyLabel *shortText = new MyLabel(this);
        shortText->setText(data->getShort());
        _shortText = shortText;
    }
    else {
        QPushButton *shortText = new MyButton(data, mainWin, data->getShort());
        _shortText = shortText;
    }
    addWidget(_shortText, 0, Qt::AlignLeft);

    if(data->isInstalled() == true) {
#ifndef NDEBUG
        std::cerr << data->getShort().toStdString() << " isInstalled:" << data->isInstalled() << std::endl;
#endif
        _checkbox->setEnabled(false);
        _shortText->setEnabled(false);
    }
    else {
        if(data->isDeselect() == false && data->isDefault() == true) {
#ifndef NDEBUG
            std::cerr << data->getShort().toStdString() << " isDeselect:" << data->isDeselect() << " isDefault:" << data->isDefault() << std::endl;
#endif
            if(data->isCheckable())
                _checkbox->setEnabled(false);
            _shortText->setEnabled(false);
        }
    }

    addStretch(2);
}

InstallItem::~InstallItem()
{
    if(_checkbox)
        _checkbox->deleteLater();
    if(_status)
        _status->deleteLater();
    if(_shortText)
        _shortText->deleteLater();
}

void InstallItem::disableCheckbox()
{
    if(_checkbox)
        _checkbox->setEnabled(false);
    _shortText->setEnabled(false);
//    _status->setEnabled(false);
}

void InstallItem::resetInstall(InstallData const *data)
{
    switch(data->getStatus()) {
    case InstallData::eInstalled:
        _status->setPixmap(QPixmap(":/images/okay"));
        break;
    case InstallData::eNeedNet:
        _status->setPixmap(QPixmap(":/images/network"));
        break;
    case InstallData::eNormal:
//        break;
        ;
    }
}

void InstallItem::setInstall()
{
    _status->setMovie(_mainWin->getInstallAnimation());
}

void InstallItem::setInstalledIcon()
{
    _status->setPixmap(QPixmap(":/images/installed"));
}

void InstallItem::setDone()
{
    _status->setPixmap(QPixmap(":/images/okay"));
}

void InstallItem::makeVisible()
{
    _mainWin->makeVisible(_status);
}

MyCheckBox::MyCheckBox(InstallPage *mainWin, InstallData const *instData, Qt::CheckState state, bool isInsensitive)
	: InsensitiveCheckBox(state, isInsensitive),
      _instData(instData),
      _mainWin(mainWin)
{
}

bool MyCheckBox::event(QEvent *event)
{
    switch(event->type()) {
    case QEvent::Enter:
#ifdef QT_KEYPAD_NAVIGATION
    case QEvent::EnterEditFocus:
#endif
    case QEvent::FocusIn:
        _mainWin->setLongDescription(_instData);

    }
    return InsensitiveCheckBox::event(event);
}

MyCheckBox::~MyCheckBox()
{
}

MyLabel::MyLabel(InstallItem *instItem)
	: QLabel(),
      _instItem(instItem)
{
}

bool MyLabel::event(QEvent *e)
{
    switch(e->type()) {
    case QEvent::Enter:
#ifdef QT_KEYPAD_NAVIGATION
    case QEvent::EnterEditFocus:
#endif
    case QEvent::FocusIn:
        _instItem->getMainWin()->setLongDescription(_instItem->getData());
        break;

//    case QEvent::MouseButtonPress:
    case QEvent::MouseButtonRelease:
    {
        MyCheckBox *checkbox = _instItem->getCheckbox();
        if(checkbox && checkbox->isEnabled() && checkbox->isInsensitive() == false)
            checkbox->toggle();
    }
    }
    return QLabel::event(e);
}

MyLabel::~MyLabel()
{
}

MyButton::MyButton(InstallData const *data, InstallPage *mainWin, QString const &label)
	: QPushButton(label),
      _data(data),
      _mainWin(mainWin)
{
    setFlat(true);
    connect(this, SIGNAL(clicked()), this, SLOT(doBrowse()));
    if(mainWin->getParentWindow()->withoutBackground() == false) {
        QPalette palette;
        QColor c(palette.color(backgroundRole()));
        c.setAlpha(127);
        palette.setBrush(backgroundRole(), QBrush(c, Qt::SolidPattern));
        setPalette(palette);
    }
    setToolTip("http://support.microsoft.com/?kbid=" + _data->getKb() + " im Browser ffnen");
}

bool MyButton::event(QEvent *event)
{
    switch(event->type()) {
    case QEvent::Enter:
#ifdef QT_KEYPAD_NAVIGATION
    case QEvent::EnterEditFocus:
#endif
    case QEvent::FocusIn:
        _mainWin->setLongDescription(_data);

    }
    return QPushButton::event(event);
}

MyButton::~MyButton()
{
}

void MyButton::doBrowse()
{
#ifndef NDEBUG
    std::cerr << "browse() " << std::endl;
#endif

#ifdef Q_OS_WIN
    QString target = "http://support.microsoft.com/?kbid=" + _data->getKb();
    ShellExecute(NULL, "open", target.toAscii(), 0, 0, SW_SHOWNORMAL);
#endif
}

#include "InstallItem.moc"
