// genericInstaller -- update windows and office
//
// Copyright (C) 2008
// Wolfgang Formann
//
// this file is part of the programm genericInstaller.  This program is
// free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your
// option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef InstallItem_h
#define InstallItem_h

#include <Qt/qboxlayout.h>
#include <Qt/qlabel.h>
#include <Qt/qpushbutton.h>
#include "InsensitiveCheckBox.h"

class InstallData;
class InstallPage;
class QEvent;
class InstallItem;
class MyLabel;

class MyCheckBox : public InsensitiveCheckBox
{
	Q_OBJECT

public:
    MyCheckBox(InstallPage *mainWin, InstallData const *instData, Qt::CheckState state, bool isInsensitive);
    ~MyCheckBox();

protected:
    bool event(QEvent * event);

private:
    InstallData const *_instData;
    InstallPage *_mainWin;
};

class MyLabel : public QLabel
{
	Q_OBJECT

public:
    MyLabel(InstallItem *instItem);
    ~MyLabel();

protected:
    bool event(QEvent * event);

private:
    InstallItem *_instItem;
};

class InstallItem : public QHBoxLayout
{
	Q_OBJECT

public:
    InstallItem(InstallPage *mainWin, InstallData const *data);
    ~InstallItem();

    bool isVisible() const {
        return _shortText->isVisible();
    }
    bool isChecked() const {
        return _checkbox == 0 ? true : _checkbox->isChecked();
    }
    InstallData const *getData() const {
        return _data;
    }
    InstallPage *getMainWin() const {
        return _mainWin;
    }
    MyCheckBox *getCheckbox() const {
        return _checkbox;
    }
    void setDone();
    void setInstall();
    void resetInstall(InstallData const *data);
    void setInstalledIcon();
    void makeVisible();
    void disableCheckbox();

private:
    MyCheckBox *_checkbox;
    MyLabel *_status;
    QWidget *_shortText;
    InstallPage *_mainWin;
    InstallData const *_data;
};

class MyButton : public QPushButton
{
	Q_OBJECT

public:
    MyButton(InstallData const *data, InstallPage *mainWin, QString const &label);
    ~MyButton();

protected:
    bool event(QEvent * event);

private slots:
    void doBrowse();

private:
    InstallData const *_data;
    InstallPage *_mainWin;
};

#endif
