// genericInstaller -- update windows and office
//
// Copyright (C) 2008
// Wolfgang Formann
//
// this file is part of the programm genericInstaller.  This program is
// free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your
// option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <assert.h>
#ifndef NDEBUG
#include <iostream>
#endif

#include "InstallPage.h"

#include "MyWindow.h"
#include "OSOfficeSwitch.h"
#include "InsensitiveCheckBox.h"
#include "InstallData.h"
#include "InstallItem.h"
#include "InstallProcess.h"
#include "DecipherProcess.h"
#include "WindowsType.h"

#include <Qt/qlayout.h>
#include <Qt/qlabel.h>
#include <Qt/qgroupbox.h>
#include <Qt/qcheckbox.h>
#include <Qt/qscrollarea.h>
#include <Qt/qscrollbar.h>
#include <Qt/qmovie.h>
#include <Qt/qprogressbar.h>
#include <Qt/qpushbutton.h>
#include <Qt/qtimer.h>
#include <Qt/qmessagebox.h>
#include <Qt/qstackedlayout.h>

#ifdef Q_OS_WIN
#	include <malloc.h>
#	include <windows.h>
#	include <io.h>
#	include <tchar.h>
#	include <wininet.h>
#endif

InstallPage::InstallPage(MyWindow *mainWindow, bool withLegend, bool withScroll)
	: WindowPage(mainWindow, true, mainWindow->getOSCheck() == 0, true),
      _headerLabel(0),
      _legend(0),
      _instPic(0),
//      _recommendedPic(0),
      _optPic(0),
      _doingPic(0),
      _doingBox(0),
      _doingLabel(0),
      _donePic(0),
      _doneBox(0),
      _doneLabel(0),
      _skipable(false),
      _didInstallSomething(false),
      _extensionLayout(0),
      _extensionList(0),
      _extensionScroll(0),
      _extensionHint(0),
      _progress2(0),
      _installingAnimation(0),
      _timer(0),
      _anyInstallable(false),
      _anyCheckable(false),
      _increment(50),
      _installDone(false),
      _withLegend(withLegend),
      _withScroll(withScroll),
      _installing(false),
      _installItems(0)
{
	QLabel *logoGraphic = new QLabel();
    if(access("logos/startlogo1.png", R_OK) == 0)
        logoGraphic->setPixmap(QPixmap("logos/startlogo1.png"));
    else
        logoGraphic->setPixmap(QPixmap(":/images/mid-logo"));
    QVBoxLayout *layoutTopLeftPage = new QVBoxLayout();
    layoutTopLeftPage->addWidget(logoGraphic, 0, Qt::AlignTop);

    if(_withLegend) {
        QGroupBox *legendBox = new QGroupBox("Symbolerklrung");
        _legend = new QGridLayout();
        legendBox->setLayout(_legend);
        _legend->setMargin(8);
        _legend->setSpacing(3);
    
        QLabel *impPic = new QLabel();
        impPic->setPixmap(QPixmap(":/images/important"));
        _legend->addWidget(impPic, 1, 0);
        QCheckBox *impBox = new InsensitiveCheckBox(Qt::Checked, true);
        _legend->addWidget(impBox, 1, 1, Qt::AlignLeft);
        QLabel *impLabel = new QLabel("Wichtige Aktualisierung");
        _legend->addWidget(impLabel, 1, 2, Qt::AlignLeft);
    
        _instPic = new QLabel();
        _instPic->setPixmap(QPixmap(":/images/okay"));
        _legend->addWidget(_instPic, 0, 0);
        QCheckBox *instBox = new InsensitiveCheckBox(Qt::Unchecked, true);
        _legend->addWidget(instBox, 0, 1, Qt::AlignLeft);
        QLabel *instLabel = new QLabel("Aktualisierung bereits\ninstalliert");
        _legend->addWidget(instLabel, 0, 2, Qt::AlignLeft);
    
        if(_parentWindow->isDataRead())
            drawCheckLegend();
    
        _doingPic = new QLabel();
//        _doingPic->setMinimumSize(22, 22);
        _legend->addWidget(_doingPic, 4, 0);
        _doingLabel = new QLabel();
        _legend->addWidget(_doingLabel, 4, 2, Qt::AlignLeft);
    
        _donePic = new QLabel();
//        _donePic->setMinimumSize(22, 22);
        _legend->addWidget(_donePic, 5, 0);
        _doneLabel = new QLabel();
        _legend->addWidget(_doneLabel, 5, 2, Qt::AlignLeft);
    
        layoutTopLeftPage->addWidget(legendBox, 1, Qt::AlignTop);
    }
    layoutTopLeftPage->addSpacing(2);

    _layoutTopPage->addLayout(layoutTopLeftPage);

    QVBoxLayout *layoutTopRightPage = new QVBoxLayout();

    // the individual part of page 2
    _headerLabel = new QLabel();
    _headerLabel->setWordWrap(true);
    _messageLabel = new QLabel();
    _messageLabel->setWordWrap(true);
    QFont f2(_headerLabel->font());
   	f2.setBold(true);
   	_messageLabel->setFont(f2);
    if(f2.pixelSize() == -1) {
        if(f2.pointSize() == -1)
            assert(0);
        f2.setPointSize(f2.pointSize() + 1);
    }
    else {
        f2.setPixelSize(f2.pixelSize() + 1);
    }
   	_headerLabel->setFont(f2);

    layoutTopRightPage->addSpacing(25);
    layoutTopRightPage->addWidget(_headerLabel, 0);
    layoutTopRightPage->addWidget(_messageLabel, 0);
    _extensionLayout = new QVBoxLayout();
    if(_withScroll) {
        _extensionList = new QWidget();
        _extensionList->setLayout(_extensionLayout);
    
        _extensionScroll = new QScrollArea();
        _extensionScroll->setWidget(_extensionList);
        layoutTopRightPage->addWidget(_extensionScroll, 1);
    }
    else {
        QGroupBox *extensionBox = new QGroupBox();
        extensionBox->setLayout(_extensionLayout);
        layoutTopRightPage->addWidget(extensionBox, 1);
    }

    _extensionHint = new QLabel();
    _extensionHint->setWordWrap(true);
    QFontMetrics fm(QFont(_extensionHint->font()));
    _extensionHint->setMinimumHeight(4 * fm.height());
    _extensionHint->setMaximumHeight(4 * fm.height());
    _extensionHint->setAlignment(Qt::AlignLeft|Qt::AlignTop);
    _extensionHint->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::MinimumExpanding);

    _progress2 = new QProgressBar();
    _progress2->setMinimum(0);
    _progress2->setMaximum(PROGRESSMAX);
    _progress2->setTextVisible(false);
    _progress2->setEnabled(false);
    _progress2->setValue(0);

    layoutTopRightPage->addWidget(_extensionHint);
    layoutTopRightPage->addWidget(_progress2);
    layoutTopRightPage->addSpacing(40);
    _layoutTopPage->addLayout(layoutTopRightPage);

    _previous = new QPushButton("&Zurck");
    connect(_previous, SIGNAL(clicked()), this, SLOT(goBack()));
    if(_parentWindow->withoutBackground() == false) {
        QPalette palette;
        QColor c(palette.color(_previous->backgroundRole()));
        c.setAlpha(127);
        palette.setBrush(_previous->backgroundRole(), QBrush(c, Qt::SolidPattern));
        _previous->setPalette(palette);
    }
    _layoutButtons->insertWidget(_layoutButtons->count() - 1, _previous);

	_next->setText("&Installieren");
	_next->setEnabled(false);
    connect(_next, SIGNAL(clicked()), this, SLOT(doInstall()));

    if(_parentWindow->withoutBackground() == false) {
        QPalette palette2;
        palette2.setBrush(_progress2->backgroundRole(), QBrush(QColor(0,0,0,0), Qt::SolidPattern));
        _progress2->setPalette(palette2);
        if(_extensionList) {
            _extensionList->setPalette(palette2);
            _extensionScroll->viewport()->setPalette(palette2);
        }

        if(_extensionList) {
            QPalette fgpalette;
            QColor c(fgpalette.color(_extensionScroll->verticalScrollBar()->backgroundRole()));
            c.setAlpha(127);
            fgpalette.setBrush(_extensionScroll->verticalScrollBar()->backgroundRole(), QBrush(c, Qt::SolidPattern));
            _extensionScroll->horizontalScrollBar()->setPalette(fgpalette);
            _extensionScroll->verticalScrollBar()->setPalette(fgpalette);
        }
    }
}

InstallPage::~InstallPage()
{
}

void InstallPage::switchToInstallLegend()
{
    if(_withLegend) {
        _instPic->setPixmap(QPixmap(":/images/installed"));

        _doingPic->setPixmap(QPixmap(":/images/installing"));
        if(_doingBox == 0) {
            _doingBox = new InsensitiveCheckBox(Qt::Checked, true);
            _legend->addWidget(_doingBox, 4, 1);
        }
        _doingLabel->setText("Installation luft");

        _donePic->setPixmap(QPixmap(":/images/okay"));
        if(_doneBox == 0) {
            _doneBox = new InsensitiveCheckBox(Qt::Checked, true);
            _legend->addWidget(_doneBox, 5, 1);
        }
        _doneLabel->setText("Installation\nist fertig");
    }
}

void InstallPage::clearInstallLegend()
{
    if(_withLegend) {
        _instPic->setPixmap(QPixmap(":/images/okay"));

        _doingPic->setPixmap(QPixmap());
        _legend->removeWidget(_doingBox);
        _doingBox->hide();
        delete _doingBox;
        _doingBox = 0;
        _doingLabel->setText("");

        _donePic->setPixmap(QPixmap());
        _legend->removeWidget(_doneBox);
        _doneBox->hide();
        delete _doneBox;
        _doneBox = 0;
        _doneLabel->setText("");
    }
}

void InstallPage::drawCheckLegend()
{
//    if(_anyCheckable == false)
//        return;

    if(_withLegend && _optPic == 0) {
//        _recommendedPic = new InsensitiveCheckBox(Qt::Unchecked, true);
//        _recommendedPic->setMinimumSize(22, 22);
//        _legend->addWidget(_recommendedPic, 2, 1);
//        QLabel *recommendedLabel = new QLabel("Auf diese Aktualisierung\nverzichten");
//        _legend->addWidget(recommendedLabel, 2, 2, Qt::AlignLeft);
    
        _optPic = new InsensitiveCheckBox(Qt::Checked, true);
//        _optPic->setMinimumSize(22, 22);
        _legend->addWidget(_optPic, 3, 1);
        QLabel *optLabel = new QLabel("Noch nicht installiert");
        _legend->addWidget(optLabel, 3, 2, Qt::AlignLeft);
    }
}

void InstallPage::fillList(std::list<InstallData *> *installItems)
{
    _extensionHint->setText("Bewegen Sie den Mauszeiger ber einen Programmnamen, um mehr darber zu erfahren.");
    if(_installItems) {
        for(std::list<InstallData *>::iterator it = _installItems->begin();
            it != _installItems->end();
            ++ it) {
    
            if((*it)->getEntry()) {
                _extensionLayout->removeItem((*it)->getEntry());
                (*it)->deleteEntry();
            }
        }
        _installItems = 0;
    }

    _installItems = installItems;
    bool anyChecked = false;
    for(std::list<InstallData *>::iterator it = _installItems->begin();
        it != _installItems->end();
        ++ it) {

        if(!(*it)->isSilent()) {
            if(!(*it)->isInstalled())
                _anyInstallable = true;
            if((*it)->isCheckable())
                _anyCheckable = true;
            (*it)->setEntry(this);
            if((*it)->getEntry()->isChecked())
                anyChecked = true;

            _extensionLayout->addLayout((*it)->getEntry());
        }
    }

    if(_extensionList)
        _extensionList->adjustSize();
    if(_withScroll == false)
        _extensionLayout->addStretch(10);

    if(anyChecked == true) {
        _next->setEnabled(true);
        _next->setText("&Installieren");
    }
    else {
        if(_skipable) {
            _next->setEnabled(true);
            _next->setText("&Weiter");
        }
        else
            _next->setEnabled(false);
    }
}

bool InstallPage::nextIsReboot()
{
    if(_didInstallSomething == false)
        return false;
    applyValue nextPackageSet = _parentWindow->getOfficeSwitch()->getNextPage(_packageSet);
    if(nextPackageSet == eXpIe7)
        return false;

    return true;
}

void InstallPage::makeVisible(QWidget *widget)
{
    if(_extensionScroll)
        _extensionScroll->ensureWidgetVisible(widget, 5, 5);
}

void InstallPage::setSkipable(bool skipable)
{
#ifndef NDEBUG
    std::cerr << "set skipable to: " << skipable << std::endl;
#endif
    _skipable = skipable;
    if(_next->isEnabled() == false) {
        _next->setEnabled(true);
        _next->setText("&Weiter");
    }
}

void InstallPage::setLongDescription(InstallData const *lPtr)
{
    if(_installing == true)
        return;

    if(!lPtr->getDesc().isEmpty())
        _extensionHint->setText(lPtr->getDesc());
    else {
        QString desc = "Hier sollte die Beschreibung fr die Erweiterung ";
        desc += lPtr->getDesc();
        desc += " stehen.";
        _extensionHint->setText(desc);
    }
}

void InstallPage::setLongDescription(QString const &msg)
{
    _extensionHint->setTextFormat(Qt::RichText);
    _extensionHint->setText(msg);
}

void InstallPage::doSelectExtension(int /*state*/)
{
    for(std::list<InstallData *>::iterator it = _installItems->begin();
        it != _installItems->end();
        ++ it) {
        if((*it)->getEntry() && (*it)->getEntry()->isVisible() && !(*it)->isInstalled() && (*it)->getEntry()->isChecked()) {
            _next->setEnabled(true);
            _next->setText("&Installieren");
            return;
        }
    }
    if(_skipable) {
        _next->setEnabled(true);
        _next->setText("&Weiter");
    }
    else
        _next->setEnabled(false);
}

QMovie *InstallPage::getInstallAnimation()
{
    switchToInstallLegend();
    if(_installingAnimation == 0) {
        _installingAnimation = new QMovie(":/images/installing");
        _installingAnimation->start();

        _progress2->setEnabled(true);
        int sumProgress = 0;
        for(std::list<InstallData *>::iterator it = _installItems->begin(); it != _installItems->end(); ++ it) {
            if((*it)->getEntry() && (*it)->getEntry()->isVisible() && !(*it)->isInstalled() && (*it)->getEntry()->isChecked()) {
                sumProgress += (*it)->getInstallTime();
            }
            else if(!(*it)->getEntry() && (*it)->isSilent()) {
                sumProgress += (*it)->getInstallTime();
            }
        }
        _progress2->setMaximum(sumProgress);
        _installTimeDone = 0;

        _timer = new QTimer(this);
        _timer->setInterval(_increment);
        _timer->setSingleShot(false);
        connect(_timer, SIGNAL(timeout()), this, SLOT(installTick()));
    }
    _timer->start();
    return _installingAnimation;
}

void InstallPage::clearInstallAnimation()
{
    clearInstallLegend();
    if(_installingAnimation) {
        _installingAnimation->stop();
        delete _installingAnimation;
        _installingAnimation = 0;
    }
    if(_timer) {
        _timer->stop();
        delete _timer;
        _timer = 0;
    }
    _progress2->setValue(_progress2->minimum());
}

void InstallPage::installTick()
{
    int value = _progress2->value();

    if(value < _installTimeDone) {
        if(_increment < 50)
            _increment += 5;
        else
            ++ _increment;
    }
    else if(value < _installTimeDone + (2 * _installTimeThis) / 3) {
        if(_increment > 50)
            -- _increment;
        else if(_increment < 50)
            ++ _increment;
    }
    else {
        if(_increment > 50)
            _increment -= 5;
        else if(_increment > 1)
            -- _increment;
    }
//#ifndef NDEBUG
//    std::cerr << "value: " << value << " incr: " << _increment << " max: " << _progress2->maximum() << " done: " << _installTimeDone << " till: " << (_installTimeDone + _installTimeThis) << std::endl;
//#endif
    value += _increment;

    if(value > _progress2->maximum() && _installDone) {
        _timer->stop();
        value = _progress2->maximum();
        _progress2->setValue(value);
        _installing = false;

        clearInstallAnimation();

#ifndef NDEBUG
        InstallDataBlock dummy;
        dummy.setApplies(_packageSet);
        std::cerr << "installTick() _packageSet: " << dummy.getAppliesString() << ", " << _packageSet << std::endl;
#endif
        if(_packageSet == eNet || _packageSet == eXpWmp11) {
            applyValue nextPackageSet = _parentWindow->getOfficeSwitch()->getNextPage(_packageSet);
#ifndef NDEBUG
            dummy.setApplies(nextPackageSet);
            std::cerr << "installTick() nextPackageSet: " << dummy.getAppliesString() << ", " << nextPackageSet << std::endl;
#endif
            if(nextPackageSet == eXpWmp11 || nextPackageSet == eXpIe7) {
                if(_didInstallSomething == false)
                    _previous->setEnabled(true);
                _parentWindow->getLayers()->setCurrentIndex(_parentWindow->getLayers()->currentIndex() - 1);
                _parentWindow->getOfficeSwitch()->doNextPage(nextPackageSet);
                return;
            }
        }
        _parentWindow->getLayers()->setCurrentIndex(1 + _parentWindow->getLayers()->currentIndex());
    }

    _progress2->setValue(value);
}

void InstallPage::goBack()
{
    _next->setEnabled(false);
    _previous->setEnabled(true);
    _parentWindow->getLayers()->setCurrentIndex(_parentWindow->getLayers()->currentIndex() - 1);
}

void InstallPage::doInstall()
{
	_next->setEnabled(false);

    bool haveToInstall = false;
    for(std::list<InstallData *>::iterator it = _installItems->begin();
        it != _installItems->end();
        ++ it) {
        if((*it)->getEntry() && (*it)->getEntry()->isVisible() && !(*it)->isInstalled() && (*it)->getEntry()->isChecked()) {
            haveToInstall = true;
            break;
        }
    }

    if(haveToInstall == false) {
        if(nextIsReboot() == true) {
            _parentWindow->getLayers()->setCurrentIndex(_parentWindow->getLayers()->currentIndex() + 1);
        }
        else {
            _parentWindow->getLayers()->setCurrentIndex(_parentWindow->getLayers()->currentIndex() - 1);
            _parentWindow->getOfficeSwitch()->nextPage(_parentWindow->getPackageSet());
        }
        return;
    }
    _previous->setEnabled(false);
    _didInstallSomething = true;
//    disconnect(_cancel, SIGNAL(clicked()), _parentWindow, SLOT(doCancel()));
//    connect(_cancel, SIGNAL(clicked()), _parentWindow, SLOT(doReboot()));

    bool needNet = false;
    for(std::list<InstallData *>::iterator it = _installItems->begin(); it != _installItems->end(); ++ it) {
        if(!(*it)->needNet())
            continue;

        if((*it)->getEntry() && (*it)->getEntry()->isVisible() && (*it)->getEntry()->isChecked()) {
            needNet = true;
            break;
        }
        else if(!(*it)->getEntry() && (*it)->isSilent() /*&& (*it)->ifNotIe7() != _ie7Found*/) {
            needNet = true;
            break;
        }
    }

    if(needNet == true) {
#ifdef Q_OS_WIN
        DWORD lpdwFlags = 0;
        DWORD dwReserved = 0;
        while(InternetGetConnectedState(&lpdwFlags, dwReserved) == FALSE) {
            switch(QMessageBox::warning(this, windowTitle(),
                                        "Die Installation einiger Komponenten bentigt den Zugriff auf das Internet.\n"
                                        "Bitte stellen Sie eine Verbindung zum Internet her.\n"
                                        "Klicken Sie danach auf Wiederholen.\n\n",
                                        "Wiederholen",
                                        "Abbrechen", 0, 0, 1)) {
            case 0: // The user clicked the Retry again button or pressed Enter
                // try again
                break;
            case 1: // The user clicked the Quit or pressed Escape
                // exit
                _next->setEnabled(true);
                return;
            }

            if((lpdwFlags & INTERNET_CONNECTION_CONFIGURED) == lpdwFlags)
                break;
        }
#endif
    }

    runInstall();
}

void InstallPage::runInstall()
{
    _installing = true;
    getInstallAnimation();
    for(std::list<InstallData *>::iterator it = _installItems->begin(); it != _installItems->end(); ++ it) {
        if(!(*it)->getEntry())
            continue;

        if((*it)->getStatus() == InstallData::eInstalled)
            (*it)->setInstalledIcon();
        (*it)->disableCheckbox();
    }

    install(0, _installItems->begin());
}

void InstallPage::installDone(InstallProcess *process)
{
    _installing = false;
    delete process;
    clearInstallAnimation();
#ifndef NDEBUG
    InstallDataBlock dummy;
    dummy.setApplies(_packageSet);
    std::cerr << "installDone() _packageSet: " << dummy.getAppliesString() << ", " << _packageSet << std::endl;
#endif
    if(_packageSet == eNet || _packageSet == eXpWmp11) {
        applyValue nextPackageSet = _parentWindow->getOfficeSwitch()->getNextPage(_packageSet);
#ifndef NDEBUG
        dummy.setApplies(nextPackageSet);
        std::cerr << "installDone() nextPackageSet: " << dummy.getAppliesString() << ", " << nextPackageSet << std::endl;
#endif
        if(nextPackageSet == eXpWmp11 || nextPackageSet == eXpIe7) {
            if(_didInstallSomething == false)
                _previous->setEnabled(true);
            _parentWindow->getLayers()->setCurrentIndex(_parentWindow->getLayers()->currentIndex() - 1);
            _parentWindow->getOfficeSwitch()->doNextPage(nextPackageSet);
            return;
        }
    }
    _parentWindow->getLayers()->setCurrentIndex(1 + _parentWindow->getLayers()->currentIndex());
}

void InstallPage::install(InstallProcess *process, std::list<InstallData *>::iterator it)
{
    if(process) {
        addInstallTime();
        delete process;
    }

    for(; it != _installItems->end(); ++ it) {
#ifndef NDEBUG
        std::cerr << "Install: " << (*it)->getPkg().toStdString() << " isInstalled: " << (*it)->isInstalled() << " Entry: " << (*it)->getEntry() << std::endl;
#endif
        if(!(*it)->isInstalled() && (*it)->getEntry() && (*it)->getEntry()->isVisible() && (*it)->getEntry()->isChecked()) {
            setInstallTime((*it)->getInstallTime());
            if((*it)->isEncrypted()) {
                DecipherProcess *p = new DecipherProcess(this, it);
                p->run();
                return;
            }

            InstallProcess *p = new InstallProcess(this, it, true);
            p->run();
            return;
        }
        else if(!(*it)->isInstalled() && !(*it)->getEntry() && (*it)->isSilent()) {
            setInstallTime((*it)->getInstallTime());
            if((*it)->isEncrypted()) {
                DecipherProcess *p = new DecipherProcess(this, it);
                p->run();
                return;
            }

            InstallProcess *p = new InstallProcess(this, it, true);
            p->run();
            return;
        }
#ifndef NDEBUG
        else
            std::cerr << "Not installing: " << (*it)->getPkg().toStdString() << std::endl;
#endif
    }
#ifndef NDEBUG
    std::cerr << "nothing more to install ..." << std::endl;
#endif
    setInstallDone();
}

#include "InstallPage.moc"
