// genericInstaller -- update windows and office
//
// Copyright (C) 2008
// Wolfgang Formann
//
// this file is part of the programm genericInstaller.  This program is
// free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your
// option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef InstallPage_h
#define InstallPage_h

#include "WindowPage.h"
#include "InstallData.h"

class QGridLayout;
class QLabel;
class QCheckBox;
class QScrollArea;
class QVBoxLayout;
class InstallData;
class QProgressBar;
class QMovie;
class QPushButton;
class InstallProcess;

class InstallPage : public WindowPage
{
	Q_OBJECT

public:
    InstallPage(MyWindow *mainWindow, bool withLegend, bool withScroll);
    ~InstallPage();

    void installDone(InstallProcess *process);
    void drawCheckLegend();
    void fillList(std::list<InstallData *> *installItems);
    void makeVisible(QWidget *widget);
    void setLongDescription(InstallData const *lPtr);
    void setLongDescription(QString const &msg);
    QMovie *getInstallAnimation();
    void clearInstallAnimation();
    void clearInstallLegend();
    void install(InstallProcess *process, std::list<InstallData *>::iterator it);

    QPushButton *getNextButton() {
        return _next;
    }
    bool haveAnyInstallable() {
        return _anyInstallable;
    }
    bool haveAnyCheckable() {
        return _anyCheckable;
    }
    void addInstallTime() {
        _installTimeDone += _installTimeThis;
    }
    void setInstallTime(int installTime) {
        _installTimeThis = installTime;
    }
    void setInstallDone() {
        _installDone = true;
    }
    void setSkipable(bool skipable);
    std::list<InstallData *> *getInstallItems() {
        return _installItems;
    }
    void setPackageSet(applyValue packageSet) {
        _packageSet = packageSet;
    }

private:
    void switchToInstallLegend();
	void runInstall();
    bool nextIsReboot();

private slots:
    void installTick();
	void doInstall();
	void goBack();

public slots:
    void doSelectExtension(int state);

protected:
    QLabel *_headerLabel;
    QLabel *_messageLabel;

private:
    applyValue _packageSet;
    QGridLayout *_legend;
    QLabel *_instPic;
//    QCheckBox *_recommendedPic;
    QCheckBox *_optPic;
    QLabel *_doingPic;
    QCheckBox *_doingBox;
    QLabel *_doingLabel;
    QLabel *_donePic;
    QCheckBox *_doneBox;
    QLabel *_doneLabel;
    bool _skipable;

    QPushButton *_previous;
    bool _didInstallSomething;

    QVBoxLayout *_extensionLayout;
    QWidget *_extensionList;
    QScrollArea *_extensionScroll;
    QLabel *_extensionHint;
    QProgressBar *_progress2;

    QMovie *_installingAnimation;
    QTimer *_timer;

    bool _anyInstallable;
    bool _anyCheckable;

    int _installTimeDone;
    int _installTimeThis;
    int _increment;
    bool _installDone;
    bool _withLegend;
    bool _withScroll;
    bool _installing;

    std::list<InstallData *> *_installItems;
};

#endif
