// genericInstaller -- update windows and office
//
// Copyright (C) 2008
// Wolfgang Formann
//
// this file is part of the programm genericInstaller.  This program is
// free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your
// option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "InstallProcess.h"

#ifndef NDEBUG
#include <iostream>
#endif
#include <fcntl.h>

#include <Qt/qtimer.h>

#include "MyWindow.h"
#include "InstallPage.h"
#include "InstallData.h"

InstallProcess::InstallProcess(InstallPage *parent, std::list<InstallData *>::iterator it, bool withWindow)
	: QProcess(),
      _it(it),
      _parent(parent),
      _minimizer(0),
      _timeout(0),
      _checkCloseWindowClass(0),
      _withWindow(withWindow)
{
}

InstallProcess::~InstallProcess()
{
    if(_minimizer != 0) {
        _minimizer->stop();
        delete _minimizer;
    }

    if(_timeout != 0) {
        _timeout->stop();
        delete _timeout;
    }

    if(_checkCloseWindowClass != 0) {
        _checkCloseWindowClass->stop();
        delete _checkCloseWindowClass;
    }
}

void InstallProcess::run()
{
#ifndef NDEBUG
    std::cerr << "Install '" << (*_it)->getShort().toStdString() << "'" << std::endl;
#endif
    if((*_it)->getEntry()) {
        QString hint = "Installiert wird: ";
        hint += "<b>" + (*_it)->getShort() + "</b>";
        _parent->setLongDescription(hint);
#ifndef NDEBUG
        std::cerr << "Throbber/v '" << (*_it)->getShort().toStdString() << "'" << std::endl;
#endif
        (*_it)->makeVisible();
        if(!(*_it)->isEncrypted())
            (*_it)->setInstall();
    }
    else if(_withWindow) {
        // show the first selected visible throbber ...
        for(std::list<InstallData *>::iterator it = _it; it != _parent->getInstallItems()->end(); ++ it) {
#ifndef NDEBUG
            std::cerr << "short desc: " << (*it)->getShort().toStdString() << std::endl;
#endif
            if((*it)->getEntry() && (*it)->getEntry()->isVisible() && !(*it)->isInstalled() && (*it)->getEntry()->isChecked()) {
                QString hint = "Installiert wird: ";
                hint += "<b>" + (*it)->getShort() + "</b>";
                _parent->setLongDescription(hint);
                (*it)->makeVisible();
                if(!(*_it)->isEncrypted())
                    (*it)->setInstall();
#ifndef NDEBUG
                std::cerr << "Throbber/i '" << (*_it)->getShort().toStdString() << "'" << std::endl;
#endif
                break;
            }
        }
    }

    if(!(*_it)->getWindowsClassToClose().isEmpty()) {
        // remember all windows of this class
#ifdef Q_OS_WIN
        HWND nw = 0;
        for(;;) {
            nw = FindWindowEx(NULL, nw, (*_it)->getWindowsClassToClose().toStdString().c_str(), NULL);
            if(nw == 0)
                break;
            _windows.push_back(nw);

#ifndef NDEBUG
           char name[256];
           int ret = GetWindowText(nw, name, sizeof(name));
           std::cerr << "Window found: " << (void *)nw << " '" << name << "'" << std::endl;
#endif
        }
#endif
    }

    QString fileToInstall;
    if((*_it)->isEncrypted()) {
#ifdef Q_OS_WIN
        fileToInstall = InstallationStatus::getInstance(_parent)->getTempDir() + '\\' + (*_it)->getPkg();
#else
        fileToInstall = InstallationStatus::getInstance(_parent)->getTempDir() + '/' + (*_it)->getPkg();
#endif
    }
    else {
#ifdef Q_OS_WIN
        fileToInstall = "packages\\" + (*_it)->getDirectory() + '\\' + (*_it)->getPkg();
#else
        fileToInstall = "packages/" + (*_it)->getDirectory() + '/' + (*_it)->getPkg();
#endif
    }

    QString command;
    QStringList args;

    if((*_it)->_getExecType() == InstallData::eTypeExe) {
        command = fileToInstall;
        args = (*_it)->getSwitches().split(" ", QString::SkipEmptyParts);
    }
    else if((*_it)->_getExecType() == InstallData::eFirefox) {
        args = (*_it)->getSwitches().split(" ", QString::SkipEmptyParts);
#ifndef NDEBUG
        std::cerr << "change filename from '" << fileToInstall.toStdString();
#endif
        char pwd[PATH_MAX + 1];
        getcwd(pwd, sizeof(pwd));
        fileToInstall = '/' + fileToInstall;
        fileToInstall = pwd + fileToInstall;
#ifdef Q_OS_WIN
        fileToInstall = "file:///" + fileToInstall;
#else
        fileToInstall = "file://" + fileToInstall;
#endif
        for(int i = 7; i < fileToInstall.length(); ++ i) {
            char c = fileToInstall.at(i).toAscii();
            if(c == '\\') {
                fileToInstall[i] = '/';
            }
            else if(c == ' ') {
                fileToInstall.replace(i, 1, "%20");
                i += 2;
            }
            else if(!(c >= 'A' && c <= 'Z' ||
                c >= 'a' && c <= 'z' ||
                c >= '0' && c <= '9' ||
                c == '_' || c == '.' || c == '/' || c == '-')) {
                char hex[4];
                sprintf(hex, "%%%02X", (unsigned char)c);
                fileToInstall.replace(i, 1, hex);
                i += 2;
            }
        }
#ifndef NDEBUG
        std::cerr << "' to '" << fileToInstall.toStdString() << "'" << std::endl;
#endif
        args << fileToInstall;
    }
    else if((*_it)->_getExecType() == InstallData::eTypeMsi) {
        command = "msiexec.exe";
        args = (*_it)->getSwitches().split(" ", QString::SkipEmptyParts);
        args << fileToInstall;
    }
    else {
        command = "wusa.exe";
        args = (*_it)->getSwitches().split(" ", QString::SkipEmptyParts);
        args << fileToInstall;
    }
    args << (*_it)->getPostSwitches().split(" ", QString::SkipEmptyParts);
    args = args.replaceInStrings("SPACE", " ", Qt::CaseSensitive);
    if(getenv("ProgramFiles") != 0)
        args = args.replaceInStrings("%ProgramFiles%", getenv("ProgramFiles"), Qt::CaseSensitive);

    setProcessChannelMode(QProcess::ForwardedChannels);
    connect(this, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(didInstall(int, QProcess::ExitStatus)));
    start(command, args);
    if(!waitForStarted(6000)) {
#ifndef NDEBUG
        std::cerr << "failed to start '" << command.toStdString() << "'" << std::endl;
#endif
        if((*_it)->getEntry()) {
            (*_it)->setDone();
#ifndef NDEBUG
            std::cerr << "De-throb/f '" << (*_it)->getShort().toStdString() << "'" << std::endl;
#endif
            if((*_it)->isEncrypted())
                ::unlink(fileToInstall.toStdString().c_str());
        }
#ifndef Q_OS_WIN
        sleep(3);
#endif
        if(_withWindow)
            _parent->install(this, ++ _it);
        else
            _parent->installDone(this);
        return;
    }

    if((*_it)->getMinimize() != 0) {
#ifndef NDEBUG
        std::cerr << "minimize: " << (*_it)->getMinimize() << " start timer " << time(0) << std::endl;
#endif
        _minimizer = new QTimer(this);
        _minimizer->setInterval((*_it)->getMinimize());
        _minimizer->setSingleShot(true);
        connect(_minimizer, SIGNAL(timeout()), this, SLOT(hideMe()));
        _minimizer->start();
    }

    if(!(*_it)->getWindowsClassToClose().isEmpty()) {
        _checkCloseWindowClass = new QTimer(this);
        _checkCloseWindowClass->setInterval(200);
        _checkCloseWindowClass->setSingleShot(false);
        connect(_checkCloseWindowClass, SIGNAL(timeout()), this, SLOT(closeWindowClass()));
        _checkCloseWindowClass->start();
    }

    if((*_it)->getIgnoreAfter() != 0) {
        _timeout = new QTimer(this);
#ifndef NDEBUG
        std::cerr << "timeout " << (*_it)->getIgnoreAfter() << std::endl;
#endif
        _timeout->setInterval((*_it)->getIgnoreAfter());
        _timeout->setSingleShot(true);
        connect(_timeout, SIGNAL(timeout()), this, SLOT(timeoutInstallation()));
        _timeout->start();
    }
}

void InstallProcess::hideMe()
{
#ifndef NDEBUG
    std::cerr << "minimize: hide " << time(0) << std::endl;
#endif
    _parent->getParentWindow()->showMinimized();
}

void InstallProcess::closeWindowClass()
{
    // iconify a newly arrived window
#ifdef Q_OS_WIN
    HWND nw = 0;
    for(;;) {
        nw = FindWindowEx(NULL, nw, (*_it)->getWindowsClassToClose().toStdString().c_str(), NULL);
        if(nw == 0)
            break;
        bool found = false;
        for(std::list<HWND>::iterator it = _windows.begin(); it != _windows.end(); ++ it) {
            if(*it == nw) {
                found = true;
                break;
            }
        }

        if(found == false) {
            if(!IsIconic(nw)) {
#ifndef NDEBUG
                char name[256];
                int ret = GetWindowText(nw, name, sizeof(name));
                std::cerr << "Minimize Window: " << (void *)nw << " '" << name << "'" << std::endl;
#endif
                SendMessage(nw, WM_SYSCOMMAND, SC_MINIMIZE, 0);
            }

            if(IsIconic(nw)) {
#ifndef NDEBUG
                char name[256];
                int ret = GetWindowText(nw, name, sizeof(name));
                std::cerr << "is mini-Window: " << (void *)nw << " '" << name << "'" << std::endl;
#endif
                // iconify only one time!
                _windows.push_back(nw);
            }
        }
    }
#endif
}

void InstallProcess::timeoutInstallation()
{
#ifndef NDEBUG
    std::cerr << "timeout reached " << (*_it)->getIgnoreAfter() << std::endl;
#endif
    didInstall(0, QProcess::NormalExit);
}

void InstallProcess::didInstall(int exitCode, QProcess::ExitStatus exitStatus)
{
//    std::cerr << "didInstall() " << exitCode << " " << exitStatus << std::endl;
    if(_minimizer != 0) {
        _minimizer->stop();
        delete _minimizer;
        _minimizer = 0;
        _parent->getParentWindow()->showNormal();
    }

    if((*_it)->isEncrypted()) {
#ifdef Q_OS_WIN
        QString fileToInstall = InstallationStatus::getInstance(_parent)->getTempDir() + '\\' + (*_it)->getPkg();
#else
        QString fileToInstall = InstallationStatus::getInstance(_parent)->getTempDir() + '/' + (*_it)->getPkg();
#endif
        ::unlink(fileToInstall.toStdString().c_str());
    }

    if(_timeout != 0) {
        _timeout->stop();
        delete _timeout;
        _timeout = 0;
    }

    if(_checkCloseWindowClass != 0) {
        _checkCloseWindowClass->stop();
        delete _checkCloseWindowClass;
        _checkCloseWindowClass = 0;
    }

    if((*_it)->getEntry()) {
        (*_it)->setDone();
#ifndef NDEBUG
        std::cerr << "De-throb/d '" << (*_it)->getShort().toStdString() << "'" << std::endl;
#endif
    }
    if(_withWindow)
        _parent->install(this, ++ _it);
    else
        _parent->installDone(this);
}

#include "InstallProcess.moc"
