// genericInstaller -- update windows and office
//
// Copyright (C) 2008
// Wolfgang Formann
//
// this file is part of the programm genericInstaller.  This program is
// free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your
// option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef InstallationStatus_h
#define InstallationStatus_h

#include <list>

#include <Qt/qstring.h>
#include <Qt/qobject.h>
#include <Qt/qxml.h>
#include <Qt/qprocess.h>

enum InstallationEnum {
    eUNSET,
    eWRONG_WINDOWS,
    eXP_WINDOWS,
    eVISTA_WINDOWS,
    eVISTA64_WINDOWS,
    eXPJUST_SP1,
    eXPNO_SP
};

class QWidget;

class MbsacliData {
public:
    MbsacliData();

    QString _kbId;
    QString _executable;
    int _severity;
};

class InstallationStatus : public QObject
{
	Q_OBJECT

private:
    InstallationStatus(QWidget *parent);

public:
    ~InstallationStatus();
	static InstallationStatus* getInstance(QWidget *parent = 0);

private:
	static InstallationStatus* instance;
    QWidget *_parent;
    void checkWindowsVersion();
    QString _servicePack;
    InstallationEnum _windowsType;

public:
    InstallationEnum getWindowsType() {
        if(_windowsType == eUNSET)
            checkWindowsVersion();
        return _windowsType;
    }
    QString const &getServicePack() {
        if(_windowsType == eUNSET)
            checkWindowsVersion();
        return _servicePack;
    }

private:
    void checkWindowsAdmin();
    bool _isAdmin;
    bool _isAdminChecked;

public:
    bool isAdmin() {
        if(_isAdminChecked == false)
            checkWindowsAdmin();
        return _isAdmin;
    }

private:
    time_t _newestTime;
    QString _tempDir;
    bool _haveOpenSsl;
    bool _openSslChecked;
    void checkOpenSsl();
    void initTempDir();
    bool checkTimes(time_t *maxTime, QString dir, bool recurse);
    int getMaxTimeIdx();
    void setMaxTime(time_t *maxTime, time_t fileTime);
public:
    bool haveOpenSsl() {
        if(_openSslChecked == false)
            checkOpenSsl();
        return _haveOpenSsl;
    }
    QString const &getTempDir() {
        if(_tempDir.isEmpty())
            initTempDir();
        return _tempDir;
    }
    bool isDateOkay();

private:
    void checkRunMbscli();
    QProcess *_mbsacli;
    bool _mbsacliDone;
    bool _mbsacliReallyDone;
    QXmlSimpleReader *_xmlAnalyzer;
    std::list<QString> _installedKBs;
    std::list<MbsacliData> _preselectKBs;

public slots:
    void didMbsacli(int exitCode, QProcess::ExitStatus exitStatus);
    void mbsacliStderr();
    void mbsacliStdout();

public:
    bool isMbsacliReallyDone() const {
        return _mbsacliReallyDone;
    }
    bool isMbsacliDone() {
        if(_mbsacli == 0 && _mbsacliDone == false)
            checkRunMbscli();
        return _mbsacliDone;
    }
    bool isOffice2003() const;
    bool isOffice2007() const;
    std::list<QString> *getInstalledKBs() {
        return &_installedKBs;
    }
    std::list<MbsacliData> *getPreselectKBs() {
        return &_preselectKBs;
    }

private:
    void checkWupdate();
    QProcess *_wupdate;
    bool _wupdateDone;
    QString _wupdateData;
    int _wupdateMinor;
    int _wupdateMajor;
public:
    bool isWupdateDone() {
        if(_wupdate == 0 && _wupdateDone == false)
            checkWupdate();
        return _wupdateDone;
    }
public slots:
    void didWupdate(int exitCode, QProcess::ExitStatus exitStatus);
    void wupdateStderr();
    void wupdateStdout();

private:
    void checkWinstall();
    QProcess *_winstall;
    bool _winstallDone;
    QString _winstallData;
    int _winstallMinor;
    int _winstallMajor;
public:
    bool isWinstallDone() {
        if(_winstall == 0 && _winstallDone == false)
            checkWinstall();
        return _winstallDone;
    }
public slots:
    void didWinstall(int exitCode, QProcess::ExitStatus exitStatus);
    void winstallStderr();
    void winstallStdout();

public:
    bool needWupdate() const;
    bool needWInstall() const;
    bool needXpForcePackage() const;
    bool needVistaForcePackage() const;

public:
    bool needXpSp3Package() const;

private:
    bool _ie7Found;
    QString _ieVersion;
public:
    bool needXpIe7Package();
    bool needXpWmp11Package() const;
    bool needVistaPreSp1Package() const;
    bool needVistaSp1Package() const;
    bool needOffice2003Sp3Package() const;
    bool needOffice2007Sp1Package() const;
    bool needNetPackage() const;
};

class AnalyzeDataHandler : public QXmlDefaultHandler
{
public:
	AnalyzeDataHandler(std::list<QString> &installedKBs, std::list<MbsacliData> &preselectKBs);
	bool fatalError(const QXmlParseException &exception);
	bool error(const QXmlParseException &exception);
	bool warning(const QXmlParseException &exception);
	bool startElement(const QString &nameSpace, const QString &lName, const QString &qName, const QXmlAttributes &attr);
    bool characters(const QString &ch);

private:
    bool _inDownloadURL;
    std::list<QString> &_installedKBs;
    std::list<MbsacliData> &_preselectKBs;
};
#endif
