// genericInstaller -- update windows and office
//
// Copyright (C) 2008
// Wolfgang Formann
//
// this file is part of the programm genericInstaller.  This program is
// free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your
// option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <assert.h>
#ifndef NDEBUG
#include <iostream>
#endif

#include "MyWindow.h"

#include "InstallProcess.h"
#include "OSCheck.h"
#include "OSOfficeSwitch.h"
#include "PackageSelectPage.h"
#include "Page3.h"
#include "DataReader.h"

#include <Qt/qapplication.h>
#include <Qt/qpalette.h>
#include <Qt/qdesktopwidget.h>
#include <Qt/qstackedlayout.h>
#include <Qt/qlabel.h>
#include <Qt/qpushbutton.h>
#include <Qt/qradiobutton.h>
#include <Qt/qfontmetrics.h>
#include <Qt/qprogressbar.h>
#include <Qt/qtimer.h>
#include <Qt/qthread.h>
#include <Qt/qgroupbox.h>
#include <Qt/qmovie.h>
#include <Qt/qdir.h>
#include <Qt/qdatetime.h>
#include <Qt/qmessagebox.h>
#include <Qt/qeventloop.h>

#ifdef Q_OS_WIN
#	include <malloc.h>
#	include <windows.h>
#	include <io.h>
#	include <tchar.h>
#	include <wininet.h>
#endif

MyWindow* MyWindow::_instance = 0;

MyWindow::MyWindow(bool withoutBackground, bool withLegend, bool withScroll)
	: QDialog(),
      _osCheckPage(0),
      _packageSelectPage(0),
      _page3(0),
      _windowsType(WindowsOther),
//      _ie7Found(false),
      _layers(0),
      _dataRead(false),
      _withoutBackground(withoutBackground)
{
    setWindowFlags(Qt::Window);

    if(_withoutBackground == false) {
        QPalette palette;
        if(access("logos/background.png", R_OK) == 0)
            palette.setBrush(backgroundRole(), QBrush(QPixmap("logos/background.png")));
        else
            palette.setBrush(backgroundRole(), QBrush(QPixmap(":/images/background")));
        this->setPalette(palette);
    }

    // size
//    QRect rect = QApplication::desktop()->screenGeometry();
//    int width = rect.width() + rect.width() / 3;
//    if(width > 1280)
//        width = 1280;
//    int height = rect.height() + rect.height() / 3;
//    if(height > 1024)
//        height = 1024;
//	resize(width * 4 / 7, height * 4 / 7);
	resize(658, 488);

    _layers = new QStackedLayout(this);
#ifndef NDEBUG
    std::cerr << "Reading start: " << std::endl;
#endif
    InstallationEnum installStatus = InstallationStatus::getInstance(this)->getWindowsType();

    if(installStatus == eVISTA_WINDOWS)
        _windowsType = WindowsVista;
    else if(installStatus == eXP_WINDOWS)
        _windowsType = WindowsXP;
    readXML();

    _layers->addWidget(_osCheckPage = new OSCheck(this));
    _layers->addWidget(_osOfficeSwitchPage = new OSOfficeSwitch(this));
    _layers->addWidget(_packageSelectPage = new PackageSelectPage(this, withLegend, withScroll));
    _layers->addWidget(_page3 = new Page3(this));
#ifndef NDEBUG
    std::cerr << "Widgets ready: " << std::endl;
#endif

    setWindowTitle("Windows-Aktualisierer");

    while(_dataRead == false) {
        QEventLoop l;
        l.processEvents(QEventLoop::ExcludeUserInputEvents);
    }

    if(_packageSelectPage->haveAnyCheckable() == false && _packageSelectPage->haveAnyInstallable() == true)
        _packageSelectPage->getNextButton()->setEnabled(true);
}

void MyWindow::readXML()
{
    _reader = new DataReader(_installItems, _rebootAction, this);
    connect(_reader, SIGNAL(finished()), this, SLOT(dataRead()));
    _reader->start();
}

MyWindow::~MyWindow()
{
}

MyWindow* MyWindow::getInstance(bool withoutBackground, bool withLegend, bool withScroll)
{
	if(_instance == 0)
		_instance = new MyWindow(withoutBackground, withLegend, withScroll);
	assert(_instance != 0);
	return _instance;
}

void MyWindow::doCancel()
{
    QDialog::close();
}

void MyWindow::doRebootAction()
{
    if(!_rebootAction.getCommand().isEmpty()) {
        QStringList args;
        args = _rebootAction.getArguments().split(" ", QString::SkipEmptyParts);
        args = args.replaceInStrings("SPACE", " ", Qt::CaseSensitive);
        QProcess *reboot = new QProcess(this);
        reboot->start(_rebootAction.getCommand(), args);
        if(!reboot->waitForStarted(6000)) {
#ifndef NDEBUG
            std::cerr << "failed to start '" << _rebootAction.getCommand().toStdString() << "'" << std::endl;
#endif
        }

        QTimer *timeout = new QTimer(this);
        timeout->setInterval(500);
        timeout->setSingleShot(true);
        connect(timeout, SIGNAL(timeout()), this, SLOT(doCancel()));
        timeout->start();
        return;
    }

    doCancel();
}

void MyWindow::browseToSp2()
{
#ifndef NDEBUG
    std::cerr << "browseToSp2()" << std::endl;
#endif

#ifdef Q_OS_WIN
    char const *target = "http://www.microsoft.com/downloads/details.aspx?familyid=049C9DBE-3B8E-4F30-8245-9E368D3CDB5A&displaylang=de";
    ShellExecute(NULL, "open", target, 0, 0, SW_SHOWNORMAL);
#endif
}

void MyWindow::browseTo(QString const &target)
{
#ifndef NDEBUG
    std::cerr << "browseTo(" << target.toStdString() << ")" << std::endl;
#endif

#ifdef Q_OS_WIN
    ShellExecute(NULL, "open", target.toAscii(), 0, 0, SW_SHOWNORMAL);
#endif
}

void MyWindow::installSp3()
{
    fillInstallPackages(eXpSp3);
    _layers->setCurrentIndex(2);
}

void MyWindow::doNext()
{
    // merge mbsacli
    InstallationStatus *iStatus = InstallationStatus::getInstance(this);
    for(std::list<InstallDataBlock *>::iterator bit = _installItems.begin(); bit != _installItems.end(); ++ bit) {
        for(std::list<InstallData *>::iterator it = (*bit)->getBlock()->begin();
            it != (*bit)->getBlock()->end();
            ++ it) {
            for(std::list<MbsacliData>::const_iterator kbIt = iStatus->getPreselectKBs()->begin(); kbIt != iStatus->getPreselectKBs()->end(); ++ kbIt) {
                if((*it)->getKb() == kbIt->_kbId) {
                    if(kbIt->_severity >= 2)
                        (*it)->setImportant();
                    (*it)->setDefault();
#ifndef NDEBUG
                    std::cerr << "pre KB: " << kbIt->_kbId.toStdString() << " exec: " << kbIt->_executable.toStdString() << " Severity: " << kbIt->_severity << std::endl;
#endif
                }
            }
            for(std::list<QString>::const_iterator kbIt = iStatus->getInstalledKBs()->begin(); kbIt != iStatus->getInstalledKBs()->end(); ++ kbIt) {
                if((*it)->getKb() == *kbIt) {
                    (*it)->setInstalled();
#ifndef NDEBUG
                std::cerr << "inst KB: " << kbIt->toStdString() << std::endl;
#endif
                }
            }
        }
    }

    // now delete every force-block
    for(std::list<InstallDataBlock *>::iterator bit = _installItems.begin(); bit != _installItems.end(); ++ bit) {
        bool haveAnyInstallable = false;
        for(std::list<InstallData *>::iterator it = (*bit)->getBlock()->begin();
            it != (*bit)->getBlock()->end();
            ++ it) {
            if(!(*it)->isInstalled())
                haveAnyInstallable = true;
        }
        // nothing to install here
        if(haveAnyInstallable == false) {
            switch((*bit)->getApplies()) {
            case eXpForce:
            case eVistaForce:
            case eXpSp3:
            case eXpIe7:
            case eXpWmp11:
            case eVistaPreSp1:
            case eVistaSp1:
            case eOffice2003Sp3:
            case eOffice2007Sp1:
            case eNet:
                // clear these blocks
                for(std::list<InstallData *>::iterator it = (*bit)->getBlock()->begin();
                    it != (*bit)->getBlock()->end();
                    ++ it) {
                    delete *it;
                    (*bit)->getBlock()->erase(it);
                    -- it;
                }

#ifndef NDEBUG
                std::cerr << "delete " << (*bit)->getAppliesString() << " since nothing is to install" << std::endl;
#endif
                delete *bit;
                _installItems.erase(bit);
                -- bit;
                break;

            case eXp:
            case eVista:
            case eOffice2003:
            case eOffice2007:
                break;
            }
        }
    }

    for(std::list<InstallDataBlock *>::iterator bit = _installItems.begin(); bit != _installItems.end(); ++ bit) {
        // check if it only contains the pre-install things ...
        if((*bit)->getApplies() == eOffice2007Sp1) {
            for(std::list<InstallData *>::iterator it = (*bit)->getBlock()->begin();
                it != (*bit)->getBlock()->end();
                ++ it) {
                if((*it)->getKb() == "936982" && (*it)->isInstalled()) {
#ifndef NDEBUG
                    std::cerr << "delete " << (*bit)->getAppliesString() << " since sp1 is already installed" << std::endl;
#endif
                    delete *bit;
                    _installItems.erase(bit);
                    break;
                }
            }
            break;
        }
    }

    _layers->setCurrentIndex(1);
//    doSelectExtension(1);
}

void MyWindow::fillInstallPackages(applyValue packageSet)
{
#ifndef NDEBUG
	InstallDataBlock dummy;
    dummy.setApplies(packageSet);
    std::cerr << "fillInstallPackages(" << dummy.getAppliesString() << ", " << packageSet << ") called" << std::endl;
#endif
    _packageSet = packageSet;
    for(std::list<InstallDataBlock *>::iterator bit = _installItems.begin(); bit != _installItems.end(); ++ bit) {
        if((*bit)->getApplies() == packageSet) {
            _packageSelectPage->setAlternateText((*bit), packageSet);
            _packageSelectPage->fillList((*bit)->getBlock());
            _packageSelectPage->drawCheckLegend();
            _page3->setAlternateText((*bit));
        }
    }
}

void MyWindow::dataRead()
{
    delete _reader;
//    _packageSelectPage->fillList(_installItems1);
//    _packageSelectPage->drawCheckLegend();
//    finishReading();
    _dataRead = true;
#ifndef NDEBUG
    std::cerr << "Reading ready: " << std::endl;
#endif
}

InstallDataBlock *MyWindow::getPackage(applyValue packageSet)
{
    for(std::list<InstallDataBlock *>::iterator bit = _installItems.begin(); bit != _installItems.end(); ++ bit) {
        if((*bit)->getApplies() == packageSet) {
            return (*bit);
        }
    }

    return false;
}        

void MyWindow::finishReading()
{
    if(_dataRead == true) {
        _osCheckPage->stopTimer();
//        connect(_osCheckPage->getNextButton(), SIGNAL(clicked()), this, SLOT(doNext()));
//        _osCheckPage->getNextButton()->setEnabled(true);
        _osOfficeSwitchPage->setAlternateText();
//        _osCheckPage->getHeaderText()->setText("Bitte klicken Sie auf \"Weiter\", um ihren Rechner zu aktualisieren.");
        doNext();
    }
}

QMovie *MyWindow::getInstallAnimation()
{
    return _packageSelectPage->getInstallAnimation();
}

void MyWindow::setLongDescription(QString const &msg)
{
    _packageSelectPage->setLongDescription(msg);
}

#include "MyWindow.moc"
