// genericInstaller -- update windows and office
//
// Copyright (C) 2008
// Wolfgang Formann
//
// this file is part of the programm genericInstaller.  This program is
// free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your
// option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#ifndef MyWindow_h
#define MyWindow_h

#include <list>
#include <Qt/qdialog.h>
#include <Qt/qprocess.h>
#include <Qt/qxml.h>

#include "InstallationStatus.h"
#include "InstallData.h"
#include "WindowsType.h"

class QStackedLayout;
class QRadioButton;
class QPushButton;
class QWidget;
class QLabel;
class QVBoxLayout;
class QHBoxLayout;
class InstallProcess;
class QProgressBar;
class QTimer;
class QThread;
class QProcess;
class QProcess;
class InstallData;
class QScrollArea;
class QCheckBox;
class QMovie;
class QGridLayout;
class OSCheck;
class OSOfficeSwitch;
class PackageSelectPage;
class Page3;

class MyWindow : public QDialog
{
	Q_OBJECT

private:
    MyWindow(bool withoutBackground, bool withLegend, bool withScroll);

public:
    ~MyWindow();
	static MyWindow* getInstance(bool withoutBackground = true, bool withLegend = false, bool withScroll = false);

private:
    void setUpInstallationStatus();
    void readXML();

private slots:
	void doCancel();
    void doRebootAction();
	void doNext();
    void dataRead();
    void installSp3();
    void browseToSp2();
    void browseTo(QString const &target);

public:
    void finishReading();
    void setLongDescription(QString const &msg);
    QMovie *getInstallAnimation();

    WindowsType getWindowsType() const {
        return _windowsType;
    }
    bool withoutBackground() const {
        return _withoutBackground;
    }
    bool isDataRead() const {
        return _dataRead;
    }
    OSCheck const *getOSCheck() const {
        return _osCheckPage;
    }
    OSOfficeSwitch *getOfficeSwitch() {
        return _osOfficeSwitchPage;
    }
    QStackedLayout *getLayers() {
        return _layers;
    }
    applyValue getPackageSet() {
        return _packageSet;
    }
    InstallDataBlock *getPackage(applyValue packageSet);
    void fillInstallPackages(applyValue packageSet);

private:
    OSCheck *_osCheckPage;
    OSOfficeSwitch *_osOfficeSwitchPage;
    PackageSelectPage *_packageSelectPage;
    Page3 *_page3;

    WindowsType _windowsType;
	static MyWindow* _instance;
    QStackedLayout *_layers;
    applyValue _packageSet;

    QThread *_reader;
    volatile bool _dataRead;
    std::list<InstallDataBlock *> _installItems;
    RebootAction _rebootAction;
    bool _withoutBackground;
};

#endif
