// genericInstaller -- update windows and office
//
// Copyright (C) 2008
// Wolfgang Formann
//
// this file is part of the programm genericInstaller.  This program is
// free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your
// option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <assert.h>

#include "OSCheck.h"
#include "InstallationStatus.h"

#include "MyWindow.h"
#include "WindowsType.h"

#include <Qt/qlayout.h>
#include <Qt/qlabel.h>
#include <Qt/qpushbutton.h>
#include <Qt/qprogressbar.h>
#include <Qt/qtimer.h>

OSCheck::OSCheck(MyWindow *mainWindow)
	: WindowLeftLargeLogoPage(mainWindow, true, true, InstallationStatus::getInstance(this)->getWindowsType() == eXPNO_SP),
      _header2Text(0),
      _progress(0),
      _upOrDown(true)
{
    // the individual part of page 1
    QLabel *headerText = 0;
    if(InstallationStatus::getInstance(this)->isDateOkay() == false) {
        headerText = new QLabel("Die enthalteten Aktualisierungen sind zu alt und knnen nicht mehr installiert werden."
                                "\n\n"
                                "Bitte beenden Sie das Programm durch einen Klick auf \"Abbrechen\".");
    }
    else if(InstallationStatus::getInstance(this)->getTempDir().isEmpty() == true) {
        headerText = new QLabel("Es konnte kein temporres Directory gefunden werden."
                                "\n\n"
                                "Bitte beenden Sie das Programm durch einen Klick auf \"Abbrechen\".");
    }
    else if(InstallationStatus::getInstance(this)->isAdmin() == false) {
        headerText = new QLabel("Dieses Programm mu mit Administratorrechten gestartet werden. "
                                "\n\n"
                                "Bitte beenden Sie das Programm durch einen Klick auf \"Abbrechen\" "
                                "und starten Sie danach das Programm als Administrator.");
    }
    else switch(InstallationStatus::getInstance(this)->getWindowsType()) {
    case eVISTA64_WINDOWS:
    case eWRONG_WINDOWS:
        headerText = new QLabel("Dieses Programm ist nur fr Windows XP oder Windows Vista geeignet. Auf Ihrem Rechner "
                                "ist eine andere Windows-Version installiert."
                                "\n\n"
                                "Bitte beenden Sie das Programm durch einen Klick auf \"Abbrechen\".");
        break;

    case eVISTA_WINDOWS:
    case eXP_WINDOWS:
        subOSCheck();
        break;

    case eXPNO_SP:
        headerText = new QLabel("Der Windows-Aktualisierer setzt mindestens Windows XP Service Pack 1 voraus. "
                                "Auf Ihrem Rechner ist kein Service Pack installiert. "
                                "Bitte installieren Sie erst Service Pack 2 und starten Sie danach den Windows-Aktualisierer erneut.");
        _next->setText("Service Pack 2 aus dem Internet laden");
        connect(_next, SIGNAL(clicked()), _parentWindow, SLOT(browseToSp2()));
        break;

    case eXPJUST_SP1:
        headerText = new QLabel("Auf Ihrem Rechner "
                                "ist " + InstallationStatus::getInstance(this)->getServicePack() + " installiert. "
                                "Bitte installieren Sie erst Service Pack 3 und starten Sie danach das Programm danach erneut."
                                "\n\n"
                                "Sie werden in wenigen Sekunden zur Installation des Service Pack 3 weitergeleitet.");
        _timer = new QTimer(this);
        _timer->setInterval(10000);
        _timer->setSingleShot(true);
        connect(_timer, SIGNAL(timeout()), _parentWindow, SLOT(installSp3()));
        _timer->start();

        break;
    }

    if(headerText != 0) {
        QFont f(headerText->font());
        f.setBold(true);
        if(f.pixelSize() == -1) {
            if(f.pointSize() == -1)
                assert(0);
            f.setPointSize(f.pointSize() + 2);
        }
        else {
            f.setPixelSize(f.pixelSize() + 2);
        }
        headerText->setFont(f);

        _layoutTopRightPage->addSpacing(90);

        headerText->setWordWrap(true);
        _layoutTopRightPage->addStretch(1);
        _layoutTopRightPage->addWidget(headerText);
        _layoutTopRightPage->addStretch(2);
    }
    // end of individual part of page 1

//	_next->setText("&Weiter");
//    _next->setEnabled(false);

    if(_parentWindow->withoutBackground() == false) {
        if(_progress) {
            QPalette palette;
            palette.setBrush(_progress->backgroundRole(), QBrush(QColor(0,0,0,0), Qt::SolidPattern));
            _progress->setPalette(palette);
        }
    }
}

OSCheck::~OSCheck()
{
}

void OSCheck::subOSCheck()
{
    _layoutTopRightPage->addSpacing(90);
    _layoutTopRightPage->addStretch(1);

    _header2Text = new QLabel("Ihre Windows-Version wird berprft, bitte haben Sie etwas "
                              "Geduld.");
    QFont f(_header2Text->font());
    f.setBold(true);
    if(f.pixelSize() == -1) {
        if(f.pointSize() == -1)
            assert(0);
        f.setPointSize(f.pointSize() + 2);
    }
    else {
        f.setPixelSize(f.pixelSize() + 2);
    }
    _header2Text->setFont(f);
    _header2Text->setWordWrap(true);
    _layoutTopRightPage->addWidget(_header2Text);
    _layoutTopRightPage->addStretch(1);

    _progress = new QProgressBar();
    _progress->setMinimum(0);
    _progress->setMaximum(PROGRESSMAX);
    _progress->setTextVisible(false);
    _progress->setValue(0);
    _layoutTopRightPage->addWidget(_progress);
    _layoutTopRightPage->addSpacing(30);

    _timer = new QTimer(this);
    _timer->setInterval(50);
    _timer->setSingleShot(false);
    connect(_timer, SIGNAL(timeout()), this, SLOT(analyseTick()));
    _timer->start();
}

void OSCheck::stopTimer()
{
    _timer->stop();
    disconnect(_timer, SLOT(timeout()));
    delete _timer;
}

void OSCheck::analyseTick()
{
    int value = _progress->value();
    if(_upOrDown) {
        if(_parentWindow->isDataRead() == true &&
           InstallationStatus::getInstance(this)->isWupdateDone() &&
           InstallationStatus::getInstance(this)->isMbsacliDone() &&
           InstallationStatus::getInstance(this)->isWinstallDone()) {
            if(value == PROGRESSMAX)
                _parentWindow->finishReading();
            else
                _timer->setInterval(1);
        }
        else {
            if(value > PROGRESSMAX * 75 / 100)
                _timer->setInterval(_timer->interval() + 10);
            else if(value > PROGRESSMAX * 50 / 100)
                _timer->setInterval(_timer->interval() + 5);
        }
        if(value == PROGRESSMAX)
            _upOrDown = false;
    }
    else {
        if(_parentWindow->isDataRead() == true &&
           InstallationStatus::getInstance(this)->isWupdateDone() &&
           InstallationStatus::getInstance(this)->isMbsacliDone() &&
           InstallationStatus::getInstance(this)->isWinstallDone())
            _parentWindow->finishReading();

        if(value == 0)
            _upOrDown = true;
    }

    if(_upOrDown)
        ++ value;
    else
        -- value;
    _progress->setValue(value);
}

#include "OSCheck.moc"
