// genericInstaller -- update windows and office
//
// Copyright (C) 2008
// Wolfgang Formann
//
// this file is part of the programm genericInstaller.  This program is
// free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your
// option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <assert.h>
#ifndef NDEBUG
#include <iostream>
#endif

#include "OSOfficeSwitch.h"

#include "MyWindow.h"

#include <Qt/qlayout.h>
#include <Qt/qlabel.h>
#include <Qt/qpushbutton.h>
#include <Qt/qradiobutton.h>
#include <Qt/qstackedlayout.h>

OSOfficeSwitch::OSOfficeSwitch(MyWindow *mainWindow)
	: WindowLeftLargeLogoPage(mainWindow, true, false, true)
{
    // the individual part of page 3
    _layoutTopRightPage->addSpacing(90);
    _layoutTopRightPage->addStretch(1);

    QLabel *message = new QLabel("Whlen Sie zwischen Aktualisierungen fr Ihr Betriebssystem oder Ihr Office-Paket.");
    message->setWordWrap(true);
    QLabel *message2 = new QLabel("Es wird empfohlen, zuerst das Betriebssystem zu aktualisieren.");
    message2->setWordWrap(true);

    _messageWindows = new QLabel("Whlen Sie diesen Punkt, um Ihr Betriebssystem zu aktualisieren.");
    _messageWindows->setWordWrap(true);
    _windowsUpdate = new QRadioButton("Windows-Aktualisierung");
    connect(_windowsUpdate, SIGNAL(clicked(bool)), this, SLOT(selectUpdate(bool)));

    _messageOffice = new QLabel("Whlen Sie diesen Punkt, um Ihr Office zu aktualisieren.");
    _messageOffice->setWordWrap(true);
    _officeUpdate = new QRadioButton("Office-Aktualisierung");
    connect(_officeUpdate, SIGNAL(clicked(bool)), this, SLOT(selectUpdate(bool)));

    QFont f(message->font());
    f.setBold(true);
    message2->setFont(f);
    _windowsUpdate->setFont(f);
    _officeUpdate->setFont(f);

    if(f.pixelSize() == -1) {
        if(f.pointSize() == -1)
            assert(0);
        f.setPointSize(f.pointSize() + 2);
    }
    else {
        f.setPixelSize(f.pixelSize() + 2);
    }
    message->setFont(f);
    _windowsUpdate->setFont(f);
    _officeUpdate->setFont(f);

    _layoutTopRightPage->addWidget(message);
    _layoutTopRightPage->addWidget(message2);
    _layoutTopRightPage->addStretch(1);
    _layoutTopRightPage->addWidget(_messageWindows);
    _layoutTopRightPage->addWidget(_windowsUpdate);
    _layoutTopRightPage->addWidget(_messageOffice);
    _layoutTopRightPage->addWidget(_officeUpdate);
    _layoutTopRightPage->addStretch(2);

	_next->setText("&Weiter");
    _next->setEnabled(false);
    connect(_next, SIGNAL(clicked()), this, SLOT(nextPage()));
}

OSOfficeSwitch::~OSOfficeSwitch()
{
}

void OSOfficeSwitch::selectUpdate(bool /*dummy*/)
{
    _next->setEnabled(true);
}

void OSOfficeSwitch::markUnneeded(InstallDataBlock *theBlock, char const *stringPart)
{
    for(std::list<InstallData *>::iterator it = theBlock->getBlock()->begin(); it != theBlock->getBlock()->end(); ++ it) {
        if((*it)->getShort().indexOf(stringPart) != -1) {
            (*it)->setInstalled();
            return;
        }
    }
}

void OSOfficeSwitch::nextPage()
{
    nextPage(eNone);
}

void OSOfficeSwitch::nextPage(applyValue SkipOverPackageSet)
{
    doNextPage(getNextPage(SkipOverPackageSet));
}

applyValue OSOfficeSwitch::getNextPage(applyValue SkipOverPackageSet)
{
#ifndef NDEBUG
	InstallDataBlock dummy;
    dummy.setApplies(SkipOverPackageSet);
    std::cerr << "getNextPage(" << dummy.getAppliesString() << ", " << SkipOverPackageSet << ") called" << std::endl;
#endif
    applyValue packageSet = eNone;
    InstallationStatus *iStatus = InstallationStatus::getInstance(this);
    InstallationEnum windowsType = iStatus->getWindowsType();
    InstallDataBlock *theBlock;

    if(windowsType == eXP_WINDOWS) {
        // check eXpForce
        if((theBlock = _parentWindow->getPackage(eXpForce)) && iStatus->needXpForcePackage()) {
            packageSet = eXpForce;
#ifndef NDEBUG
            std::cerr << "getNextPage() XpForce step" << std::endl;
#endif
            if(iStatus->needWupdate() == false) {
                markUnneeded(theBlock, "Update Agent");
                // mark as unneeded
            }
            if(iStatus->needWInstall() == false) {
                markUnneeded(theBlock, "Installer");
                // mark as unneeded
            }
        }
    }

    if(packageSet == SkipOverPackageSet)
        packageSet = eNone;

    if(packageSet == eNone && windowsType == eXP_WINDOWS && _windowsUpdate->isChecked()) {
        // check eXpSp3
        if((theBlock = _parentWindow->getPackage(eXpSp3)) && iStatus->needXpSp3Package()) {
            packageSet = eXpSp3;
#ifndef NDEBUG
            std::cerr << "getNextPage() XpSP3 step" << std::endl;
#endif
        }
        if(packageSet == SkipOverPackageSet)
            packageSet = eNone;

        if(packageSet == eNone) {
            // check eNet
            if((theBlock = _parentWindow->getPackage(eNet)) && iStatus->needNetPackage()) {
                packageSet = eNet; // eventuell kein Reboot
#ifndef NDEBUG
                std::cerr << "getNextPage() Net step" << std::endl;
#endif
                if(SkipOverPackageSet == eNet || SkipOverPackageSet == eXpWmp11 || SkipOverPackageSet == eXpIe7)
                    packageSet = eNone;
            }
        }

        if(packageSet == eNone) {
            // check eXpWmp11
            if((theBlock = _parentWindow->getPackage(eXpWmp11)) && iStatus->needXpWmp11Package()) {
                packageSet = eXpWmp11; // eventuell kein Reboot
#ifndef NDEBUG
                std::cerr << "getNextPage() XpWmp11 step" << std::endl;
#endif
                if(SkipOverPackageSet == eXpWmp11 || SkipOverPackageSet == eXpIe7)
                    packageSet = eNone;
            }
        }

        // check eXpIe7
        if(packageSet == eNone && (theBlock = _parentWindow->getPackage(eXpIe7)) && iStatus->needXpIe7Package()) {
            packageSet = eXpIe7; // immer Reboot
#ifndef NDEBUG
            std::cerr << "getNextPage() XpIE7 step" << std::endl;
#endif
        }
        if(packageSet == SkipOverPackageSet)
            packageSet = eNone;
    }

    if(windowsType == eVISTA_WINDOWS) {
        // check eVistaForce
        if((theBlock = _parentWindow->getPackage(eVistaForce)) && iStatus->needVistaForcePackage()) {
            packageSet = eVistaForce;
#ifndef NDEBUG
            std::cerr << "getNextPage() VistaForce step" << std::endl;
#endif
            if(iStatus->needWupdate() == false) {
                markUnneeded(theBlock, "Update Agent");
                // mark as unneeded
            }
            if(iStatus->needWInstall() == false) {
                markUnneeded(theBlock, "Installer");
            }
            if(packageSet == SkipOverPackageSet)
                packageSet = eNone;
        }
    }

    if(packageSet == eNone && windowsType == eVISTA_WINDOWS && _windowsUpdate->isChecked()) {
        // check eVistaSp1
        if((theBlock = _parentWindow->getPackage(eVistaSp1)) && iStatus->needVistaSp1Package()) {
            // check eVistaPreSp1
            if((theBlock = _parentWindow->getPackage(eVistaPreSp1)) && iStatus->needVistaPreSp1Package()) {
                if(eVistaPreSp1 == SkipOverPackageSet) {
                    packageSet = eVistaSp1;
#ifndef NDEBUG
                    std::cerr << "getNextPage() VistaSp1/1 step" << std::endl;
#endif
                }
                else {
                    packageSet = eVistaPreSp1;
#ifndef NDEBUG
                    std::cerr << "getNextPage() VistaPreSp1 step" << std::endl;
#endif
                }
            }
            else {
                packageSet = eVistaSp1;
#ifndef NDEBUG
                std::cerr << "getNextPage() VistaSp1/2 step" << std::endl;
#endif
            }
            if(packageSet == SkipOverPackageSet)
                packageSet = eNone;
        }
    }

    if(packageSet == eNone && _officeUpdate->isChecked()) {
        if(iStatus->isOffice2003()) {
            // check eOffice2003Sp3
            if((theBlock = _parentWindow->getPackage(eOffice2003Sp3)) && iStatus->needOffice2003Sp3Package()) {
                packageSet = eOffice2003Sp3;
#ifndef NDEBUG
                std::cerr << "getNextPage() VistaSp1/2 step" << std::endl;
#endif
                if(packageSet == SkipOverPackageSet)
                    packageSet = eNone;
            }
        }
        else {
            // check eOffice2007Sp1
            if((theBlock = _parentWindow->getPackage(eOffice2007Sp1)) && iStatus->needOffice2007Sp1Package()) {
                packageSet = eOffice2007Sp1;
#ifndef NDEBUG
                std::cerr << "getNextPage() VistaSp1/2 step" << std::endl;
#endif
                if(packageSet == SkipOverPackageSet)
                    packageSet = eNone;
            }
        }
    }

    if(windowsType == eVISTA_WINDOWS || _officeUpdate->isChecked()) {
        if(packageSet == eNone) {
            // check eNet
            if((theBlock = _parentWindow->getPackage(eNet)) && iStatus->needNetPackage()) {
                packageSet = eNet; // immer reboot
#ifndef NDEBUG
                std::cerr << "getNextPage() Net/2/2 step" << std::endl;
#endif
                if(packageSet == SkipOverPackageSet)
                    packageSet = eNone;
            }
        }
    }

    if(packageSet == eNone && _windowsUpdate->isChecked()) {
        if(windowsType == eXP_WINDOWS) {
            packageSet = eXp;
#ifndef NDEBUG
            std::cerr << "getNextPage() Xp step" << std::endl;
#endif
        }
        else {
            packageSet = eVista;
#ifndef NDEBUG
            std::cerr << "getNextPage() Vista step" << std::endl;
#endif
        }
        if(packageSet == SkipOverPackageSet)
            packageSet = eNone;
    }

    if(packageSet == eNone && _officeUpdate->isChecked()) {
        if(iStatus->isOffice2003()) {
            packageSet = eOffice2003;
#ifndef NDEBUG
            std::cerr << "getNextPage() Office 2003 step" << std::endl;
#endif
        }
        else {
            packageSet = eOffice2007;
#ifndef NDEBUG
            std::cerr << "getNextPage() Office 2007 step" << std::endl;
#endif
        }
        if(packageSet == SkipOverPackageSet)
            packageSet = eNone;
    }

#ifndef NDEBUG
    dummy.setApplies(packageSet);
    std::cerr << "getNextPage() returns " << dummy.getAppliesString() << std::endl;
#endif
    return packageSet;
}

void OSOfficeSwitch::doNextPage(applyValue packageSet)
{
#ifndef NDEBUG
	InstallDataBlock dummy;
    dummy.setApplies(packageSet);
    std::cerr << "doNextPage(" << dummy.getAppliesString() << ", " << packageSet << ") called" << std::endl;
#endif
    _parentWindow->fillInstallPackages(packageSet);
    _parentWindow->getLayers()->setCurrentIndex(1 + _parentWindow->getLayers()->currentIndex());
}

void OSOfficeSwitch::setAlternateText()
{
    InstallationEnum windowsType = InstallationStatus::getInstance(this)->getWindowsType();
    if(windowsType == eXP_WINDOWS)
        _windowsUpdate->setText("Windows-XP-Aktualisierung");
    else
        _windowsUpdate->setText("Windows-Vista-Aktualisierung");

    if(InstallationStatus::getInstance(this)->isOffice2003())
        _officeUpdate->setText("Office-2003-Aktualisierung");
    else if(InstallationStatus::getInstance(this)->isOffice2007())
        _officeUpdate->setText("Office-2007-Aktualisierung");
    else {
        if(InstallationStatus::getInstance(this)->isMbsacliReallyDone() == false)
            _messageOffice->setText("Mglicherweise mssen zunchst einige Systemkomponenten auf den neuesten Stand gebracht werden,"
                                    " bevor das Office-Paket aktualisiert werden kann.");
        _officeUpdate->setEnabled(false);
        _officeUpdate->setText("Office-Aktualisierung");
    }
}

#include "OSOfficeSwitch.moc"
