// genericInstaller -- update windows and office
//
// Copyright (C) 2008
// Wolfgang Formann
//
// this file is part of the programm genericInstaller.  This program is
// free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your
// option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "WindowPage.h"

#include "AboutProgramm.h"
#include "MyWindow.h"

#include <Qt/qlayout.h>
#include <Qt/qpushbutton.h>

WindowPage::WindowPage(MyWindow *mainWindow, bool abortButton, bool aboutButton, bool nextButton)
	: QWidget(),
      _parentWindow(mainWindow),
      _layoutButtons(0),
      _cancel(0),
      _about(0),
      _next(0)
{
    _layoutThePage = new QVBoxLayout();
    setLayout(_layoutThePage);
    _layoutTopPage = new QHBoxLayout();

    _layoutThePage->addLayout(_layoutTopPage);

    if(abortButton || aboutButton || nextButton) {
        _layoutButtons = new QHBoxLayout();

        if(abortButton) {
            _cancel = new QPushButton("A&bbrechen", this);
            connect(_cancel, SIGNAL(clicked()), _parentWindow, SLOT(doCancel()));
            _layoutButtons->addWidget(_cancel);
        }

        if(aboutButton) {
            _about = new QPushButton("ber", this);
            connect(_about, SIGNAL(clicked()), this, SLOT(doAbout()));
            _layoutButtons->addWidget(_about);
        }

        _layoutButtons->addStretch(1);

        if(nextButton) {
            _next = new QPushButton("", this);
            _next->setDefault(true);
            _layoutButtons->addWidget(_next);
        }

        _layoutThePage->addLayout(_layoutButtons, 0);

        if(_parentWindow->withoutBackground() == false) {
            QPalette palette;
            QColor c(palette.color((_next ? _next : _about ? _about : _cancel)->backgroundRole()));
            c.setAlpha(127);
            palette.setBrush((_next ? _next : _about ? _about : _cancel)->backgroundRole(), QBrush(c, Qt::SolidPattern));
            if(_next)
                _next->setPalette(palette);
            if(_about)
                _about->setPalette(palette);
            if(_cancel)
                _cancel->setPalette(palette);
        }
    }
}

WindowPage::~WindowPage()
{
}

void WindowPage::doAbout()
{
	AboutProgramm aboutDialog(_parentWindow);
	aboutDialog.show();
	aboutDialog.exec();
}

#include "WindowPage.moc"
