// genericInstaller -- update windows and office
//
// Copyright (C) 2008
// Wolfgang Formann
//
// this file is part of the programm genericInstaller.  This program is
// free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your
// option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <getopt.h>
#include <iostream>
#include <list>

#include <Qt/qapplication.h>
#include <Qt/qtranslator.h>
#include <Qt/qlocale.h>
#include <Qt/qicon.h>
#include <Qt/qsettings.h>

#include "genInst/MyWindow.h"

static const struct option options[] = {
    { "help",          no_argument,       0, 'h'                },
    { "nobackground",  no_argument,       0, 'b'                },
    { "withlegend",    no_argument,       0, 'l'                },
    { "withscroll",    no_argument,       0, 's'                },
    { 0,               0,                 0, 0                  }
};

static void usage(char const *progname);

int main(int argc, char **argv)
{
	QApplication app(argc, argv);
	QTranslator transPatch;
	transPatch.load(":/text/genInstall_de.ts");
	app.installTranslator(&transPatch);

	QApplication::setWindowIcon(QIcon(":/images/prog-icon"));
	app.connect(&app, SIGNAL(lastWindowClosed()), &app, SLOT(quit()));

    int opt;
    bool nobackground = false;
    bool withLegend = true;
    bool withScroll = true;

    while((opt = getopt_long(argc, argv, "hb1ls", options, 0)) != -1) {
        switch(opt) {
        case 'b':
            nobackground = true;
            break;

        case 'l':
            withLegend = true;
            break;

        case 's':
            withScroll = true;
            break;

        case 'h':
            usage(argv[0]);
            exit(0);

		default:
            usage(argv[0]);
            exit(1);
        }
    }

    MyWindow *theWindow = MyWindow::getInstance(nobackground, withLegend, withScroll);
	theWindow->show();
	app.exec();

    exit(0);
}

static void usage(char const *progname)
{
    std::cerr << QObject::tr("Usage", "usage").toStdString();
	    std::cerr << ": " << progname << " ";
	    std::cerr << QObject::tr("option", "usage").toStdString();
	    std::cerr << std::endl;
    std::cerr << "\t--help         -h              ";
	    std::cerr << QObject::tr("this output", "usage").toStdString();
	    std::cerr << std::endl;
    std::cerr << "\t--nobackground -b              ";
	    std::cerr << QObject::tr("do not use a background image", "usage").toStdString();
	    std::cerr << std::endl;
    std::cerr << "\t--withlegend   -l              ";
	    std::cerr << QObject::tr("show legend of symbols", "usage").toStdString();
	    std::cerr << std::endl;
    std::cerr << "\t--withscroll   -s              ";
	    std::cerr << QObject::tr("show scrollable area", "usage").toStdString();
	    std::cerr << std::endl;
}
